@@ -241,8 +241,6 @@ date_ops = ('lt', 'gt', 'in_range', 'date_spec')
 date_spec_names = '''hours monthdays weekdays yearsdays months \
 weeks years weekyears moon'''.split()
 in_range_attrs = ('start', 'end')
-roles_names = ('Stopped', 'Started', 'Master', 'Slave')
-actions_names = ('start', 'promote', 'demote', 'stop')
 node_default_type = "normal"
 node_attributes_keyw = ("attributes", "utilization")
 shadow_envvar = "CIB_shadow"
@@ -490,7 +488,8 @@ QDEVICE_HELP_INFO = """  QDevice participates in quorum decisions. With the assi
   and highly recommended for 2 node clusters."""
 
 
-SSH_OPTION = "-o StrictHostKeyChecking=no"
+SSH_OPTION_ARGS = ["-o", "StrictHostKeyChecking=no"]
+SSH_OPTION = ' '.join(SSH_OPTION_ARGS)
 
 
 CLOUD_AWS = "amazon-web-services"
@@ -522,4 +521,16 @@ HA_GROUP = "haclient"
 SCHEMA_MIN_VER_SUPPORT_OCF_1_1 = "pacemaker-3.7"
 REJOIN_COUNT = 60
 REJOIN_INTERVAL = 10
+DC_DEADTIME_DEFAULT = 20
+
+ADVISED_ACTION_LIST = ['monitor', 'start', 'stop', 'promote', 'demote']
+ADVISED_KEY_LIST = ['timeout', 'interval', 'role']
+DEFAULT_INTERVAL_IN_ACTION = "20s"
+WAIT_TIMEOUT_MS_DEFAULT = 120000
+CSYNC2_SERVICE = "csync2.socket"
+
+RSC_ROLE_PROMOTED = "Promoted"
+RSC_ROLE_UNPROMOTED = "Unpromoted"
+RSC_ROLE_PROMOTED_LEGACY = "Master"
+RSC_ROLE_UNPROMOTED_LEGACY = "Slave"
 # vim:ts=4:sw=4:et:
