@@ -8,6 +8,7 @@ configuration file, and also the corosync-* utilities.
 import os
 import re
 import socket
+
 from . import utils
 from . import tmpfiles
 from . import parallax
@@ -62,7 +63,7 @@ def query_ring_status():
     rc, out, err = utils.get_stdout_stderr("corosync-cfgtool -s")
     if rc != 0 and err:
         raise ValueError(err)
-    if rc == 0 and out:
+    if out:
         print(out)
 
 
@@ -97,6 +98,7 @@ def query_qnetd_status():
     """
     Query qnetd status
     """
+    import crmsh.bootstrap  # workaround for circular dependencies
     if not utils.is_qdevice_configured():
         raise ValueError("QDevice/QNetd not configured!")
     cluster_name = get_value('totem.cluster_name')
@@ -107,11 +109,10 @@ def query_qnetd_status():
         raise ValueError("host for qnetd not configured!")
 
     # Configure ssh passwordless to qnetd if detect password is needed
-    if utils.check_ssh_passwd_need(qnetd_addr):
-        print("Copy ssh key to qnetd node({})".format(qnetd_addr))
-        rc, _, err = utils.get_stdout_stderr("ssh-copy-id -i /root/.ssh/id_rsa.pub root@{}".format(qnetd_addr))
-        if rc != 0:
-            raise ValueError(err)
+    local_user, remote_user = utils.user_pair_for_ssh(qnetd_addr)
+    if utils.check_ssh_passwd_need(local_user, remote_user, qnetd_addr):
+        crmsh.bootstrap.configure_ssh_key(local_user)
+        utils.ssh_copy_id(local_user, remote_user, qnetd_addr)
 
     cmd = "corosync-qnetd-tool -lv -c {}".format(cluster_name)
     result = parallax.parallax_call([qnetd_addr], cmd)
