@@ -19,12 +19,12 @@ class ConsoleCustomHandler(logging.StreamHandler):
     """
     A custom handler for console
 
-    Redirect ERROR message to sys.stderr
-    Redirect INFO/WARNING/DEBUG message to sys.stdout
+    Redirect ERROR/WARNING/DEBUG message to sys.stderr
+    Redirect INFO message to sys.stdout
     """
 
     def emit(self, record):
-        if record.levelno < logging.ERROR:
+        if record.levelno == logging.INFO:
             stream = sys.stdout
         else:
             stream = sys.stderr
@@ -46,14 +46,11 @@ class ConsoleCustomFormatter(logging.Formatter):
         "ERROR": constants.RED
     }
     FORMAT = "%(levelname)s: %(message)s"
-    FORMAT_RAW = "%(message)s"
 
-    def __init__(self, lineno=-1, raw_msg=False, fmt=None):
+    def __init__(self, lineno=-1, fmt=None):
         self.lineno = lineno
         if fmt:
             super().__init__(fmt=fmt)
-        elif raw_msg:
-            super().__init__(fmt=self.FORMAT_RAW)
         else:
             super().__init__(fmt=self.FORMAT)
 
@@ -105,6 +102,24 @@ class DebugCustomFilter(logging.Filter):
             return True
 
 
+class GroupWriteRotatingFileHandler(logging.handlers.RotatingFileHandler):
+    """
+    A custom rotating file handler which keeps log files group wirtable after rotating
+    Source: https://stackoverflow.com/a/6779307
+    """
+    def _open(self):
+        rtv = super()._open()
+        try:
+            shutil.chown(rtv.name, group=constants.HA_GROUP)
+            os.fchmod(rtv.fileno(), 0o664)
+            shutil.chown(rtv.name, user=constants.HA_USER)
+        except PermissionError:
+            # The file has been open, and FileHandler can write to it.
+            # Failing to change owner or mode is not a fatal error.
+            pass
+        return rtv
+
+
 LOGGING_CFG = {
     "version": 1,
     "disable_existing_loggers": "False",
@@ -144,7 +159,7 @@ LOGGING_CFG = {
             "flushLevel": logging.CRITICAL,
         },
         "file": {
-            "class": "logging.handlers.RotatingFileHandler",
+            "()": GroupWriteRotatingFileHandler,
             "filename": CRMSH_LOG_FILE,
             "formatter": "file",
             "filters": ["filter"],
@@ -245,18 +260,6 @@ class LoggerUtils(object):
             sys.stdout.flush()
             console_handler.terminator = "\n"
 
-    @contextmanager
-    def raw_msg(self):
-        """
-        Only print raw message, without levelname
-        """
-        console_handler = self.get_handler("console")
-        try:
-            console_handler.setFormatter(ConsoleCustomFormatter(raw_msg=True))
-            yield
-        finally:
-            console_handler.setFormatter(ConsoleCustomFormatter())
-
     @contextmanager
     def only_file(self):
         """
@@ -270,8 +273,12 @@ class LoggerUtils(object):
             self.logger.addHandler(console_handler)
 
     def log_only_to_file(self, msg, level=logging.INFO):
-        with self.only_file():
-            self.logger.log(level, msg)
+        from .config import core
+        if core.debug:
+            self.logger.log(logging.DEBUG, msg)
+        else:
+            with self.only_file():
+                self.logger.log(level, msg)
 
     @contextmanager
     def buffer(self):
@@ -311,38 +318,29 @@ class LoggerUtils(object):
         finally:
             self.reset_lineno(self.__save_lineno)
 
-    def status_done(self):
-        """
-        Log info message "done" in console
-        """
-        with self.raw_msg():
-            self.logger.info("done")
-
     @contextmanager
     def status_long(self, msg):
         """
-        To wait and mark something finished, start with msg, end of done
+        To wait and mark something finished, start with BEGIN msg, end of END msg
         """
-        with self.suppress_new_line():
-            self.logger.info("%s...", msg)
+        self.logger.info("BEGIN %s", msg)
         try:
-            yield
-        except:
-            print("")
+            yield ProgressBar()
+        except Exception:
+            self.logger.error("FAIL %s", msg)
             raise
         else:
-            self.status_done()
+            self.logger.info("END %s", msg)
 
     def wait_input(self, prompt_string, default=""):
         """
         Wrap input function with recording prompt string and input result
         """
-        with self.suppress_new_line():
+        with self.only_file():
             self.logger.info(prompt_string)
-        value = input()
+        value = input(prompt_string)
         if not value:
             value = default
-            print()
         with self.only_file():
             self.logger.info("input result: %s", value)
         return value
@@ -423,12 +421,20 @@ class LoggerUtils(object):
             self.logger.info("offending xml: %s", xml)
 
 
-def setup_directory_for_logfile():
-    """
-    Create log file's parent directory
-    """
-    _dir = os.path.dirname(CRMSH_LOG_FILE)
-    os.makedirs(_dir, exist_ok=True)
+class ProgressBar:
+    def __init__(self):
+        self._i = 0
+
+    def progress(self):
+        try:
+            width, _ = os.get_terminal_size()
+        except OSError:
+            # not a terminal
+            return
+        self._i = (self._i + 1) % width
+        line = '\r{}{}'.format('.' * self._i, ' ' * (width - self._i))
+        sys.stdout.write(line)
+        sys.stdout.flush()
 
 
 def setup_logging(only_help=False):
@@ -438,11 +444,14 @@ def setup_logging(only_help=False):
     # To avoid the potential "permission denied" error under other users (boo#1192754)
     if only_help:
         LOGGING_CFG["handlers"]["file"] = {'class': 'logging.NullHandler'}
-    else:
-        setup_directory_for_logfile()
+    # dirname(CRMSH_LOG_FILE) should be created by package manager during installation
+    try:
+        with open(CRMSH_LOG_FILE, 'a'):
+            pass
+    except (PermissionError, FileNotFoundError) as e:
+        print('{}WARNING:{} Failed to open log file: {}'.format(constants.YELLOW, constants.END, e), file=sys.stderr)
+        LOGGING_CFG["handlers"]["file"] = {'class': 'logging.NullHandler'}
     logging.config.dictConfig(LOGGING_CFG)
-    if os.path.exists(CRMSH_LOG_FILE):
-        shutil.chown(CRMSH_LOG_FILE, constants.HA_USER, constants.HA_GROUP)
 
 
 def setup_logger(name):
