@@ -11,6 +11,7 @@ from . import options
 from . import constants
 from . import clidisplay
 from . import term
+from . import upgradeutil
 from . import utils
 from . import userdir
 
@@ -157,12 +158,6 @@ def set_interactive():
         options.interactive = True
 
 
-def compatibility_setup():
-    if not utils.is_pcmk_118():
-        del constants.attr_defaults["node"]
-        constants.cib_no_section_rc = 22
-
-
 def add_quotes(args):
     '''
     Add quotes if there's whitespace in one of the
@@ -249,7 +244,6 @@ def main_input_loop(context, user_args):
     Main input loop for crmsh. Parses input
     line by line.
     """
-    compatibility_setup()
     rc = handle_noninteractive_use(context, user_args)
     if rc is not None:
         return rc
@@ -366,15 +360,19 @@ def run():
             options.batch = True
         user_args = parse_options()
         if config.core.debug:
-            print(utils.debug_timestamp())
+            logger.debug(utils.debug_timestamp())
         term.init()
         if options.profile:
             return profile_run(context, user_args)
         else:
+            upgradeutil.upgrade_if_needed()
             return main_input_loop(context, user_args)
     except KeyboardInterrupt:
-        print("Ctrl-C, leaving")
-        sys.exit(1)
+        if config.core.debug:
+            raise
+        else:
+            print("Ctrl-C, leaving")
+            sys.exit(1)
     except ValueError as e:
         if config.core.debug:
             import traceback
