@@ -301,7 +301,7 @@ e.g. crm cluster init ocfs2 -o <ocfs2_device>
 
         no_quorum_policy_value = utils.get_property("no-quorum-policy")
         if not no_quorum_policy_value or no_quorum_policy_value != "freeze":
-            utils.set_property(no_quorum_policy="freeze")
+            utils.set_property("no-quorum-policy", "freeze")
             logger.info("  'no-quorum-policy' is changed to \"freeze\"")
 
         if self.use_cluster_lvm2:
