@@ -3,7 +3,12 @@
 
 
 import os
+
 import parallax
+import crmsh.utils
+from crmsh import userdir
+
+Error = parallax.Error
 
 
 class Parallax(object):
@@ -18,6 +23,7 @@ class Parallax(object):
         self.nodes = nodes
         self.askpass = askpass
         self.ssh_options = ssh_options
+        self._sudoer = None
         self.strict = strict
 
         # used for call
@@ -34,7 +40,14 @@ class Parallax(object):
     def prepare(self):
         opts = parallax.Options()
         if self.ssh_options is None:
-            self.ssh_options = ['StrictHostKeyChecking=no', 'ConnectTimeout=10']
+            self.ssh_options = ['StrictHostKeyChecking=no',
+                    'ConnectTimeout=10',
+                    'LogLevel=error']
+        sudoer, _ = crmsh.utils.user_pair_for_ssh(self.nodes[0])
+        if sudoer is not None:
+            # FIXME: this is really unreliable
+            self.ssh_options.append('IdentityFile={}/.ssh/id_rsa'.format(userdir.gethomedir(sudoer)))
+            self._sudoer = sudoer
         opts.ssh_options = self.ssh_options
         opts.askpass = self.askpass
         # warn_message will available from parallax-1.0.5
@@ -45,14 +58,19 @@ class Parallax(object):
 
     def handle(self, results):
         for host, result in results:
-            if isinstance(result, parallax.Error):
-                exception_str = "Failed on {}: {}".format(host, result)
-                if self.strict:
-                    raise ValueError(exception_str)
+            if isinstance(result, parallax.Error) and self.strict:
+                raise ValueError("Failed on {}: {}".format(host, result))
         return results
 
     def call(self):
-        results = parallax.call(self.nodes, self.cmd, self.opts)
+        host_port_user = []
+        for host in self.nodes:
+            _, remote_user = crmsh.utils.user_pair_for_ssh(host)
+            host_port_user.append([host, None, remote_user])
+        # FIXME: this is really unreliable
+        sudoer = userdir.get_sudoer()
+        cmd = f'sudo bash -c "{self.cmd}"' if sudoer else self.cmd
+        results = parallax.call(host_port_user, cmd, self.opts)
         return self.handle(list(results.items()))
 
     def slurp(self):
@@ -63,11 +81,19 @@ class Parallax(object):
     def copy(self):
         results = parallax.copy(self.nodes, self.src, self.dst, self.opts)
         return self.handle(list(results.items()))
+    def run(self):
+        sudoer = userdir.get_sudoer()
+        cmd = f'sudo bash -c "{self.cmd}"' if sudoer else self.cmd
+        host_port_user = []
+        for host in self.nodes:
+            _, remote_user = crmsh.utils.user_pair_for_ssh(host)
+            host_port_user.append([host, None, remote_user])
+        return parallax.run(host_port_user, cmd, self.opts)
 
 
 def parallax_call(nodes, cmd, askpass=False, ssh_options=None, strict=True):
     """
-    Executes the given command on a set of hosts, collecting the output
+    Executes the given command on a set of hosts, collecting the output, and raise exception when error occurs
     nodes:       a set of hosts
     cmd:         command
     askpass:     Ask for a password if passwordless not configured
@@ -105,3 +131,15 @@ def parallax_copy(nodes, src, dst, askpass=False, ssh_options=None, strict=True)
     """
     p = Parallax(nodes, src=src, dst=dst, askpass=askpass, ssh_options=ssh_options, strict=strict)
     return p.copy()
+
+def parallax_run(nodes, cmd, askpass=False, ssh_options=None, strict=True):
+    """
+    Executes the given command on a set of hosts, collecting the output and any error
+    nodes:       a set of hosts
+    cmd:         command
+    askpass:     Ask for a password if passwordless not configured
+    ssh_options: Extra options to pass to SSH
+    Returns [(host, (rc, stdout, stdin)), ...] or ValueError exception
+    """
+    p = Parallax(nodes, cmd=cmd, askpass=askpass, ssh_options=ssh_options, strict=strict)
+    return p.run()
