@@ -2,7 +2,11 @@ import os
 import re
 import socket
 import functools
+import subprocess
+import tempfile
+import typing
 from enum import Enum
+from . import constants
 from . import utils
 from . import parallax
 from . import corosync
@@ -26,7 +30,7 @@ class QdevicePolicy(Enum):
     QDEVICE_RESTART_LATER = 2
 
 
-def evaluate_qdevice_quorum_effect(mode, diskless_sbd=False):
+def evaluate_qdevice_quorum_effect(mode, diskless_sbd=False, is_stage=False):
     """
     While adding/removing qdevice, get current expected votes and actual total votes,
     to calculate after adding/removing qdevice, whether cluster has quorum
@@ -43,6 +47,9 @@ def evaluate_qdevice_quorum_effect(mode, diskless_sbd=False):
     if utils.calculate_quorate_status(expected_votes, actual_votes) and not diskless_sbd:
         # safe to use reload
         return QdevicePolicy.QDEVICE_RELOAD
+    elif mode == QDEVICE_ADD and not is_stage:
+        # Add qdevice from init process, safe to restart
+        return QdevicePolicy.QDEVICE_RESTART
     elif xmlutil.CrmMonXmlParser.is_any_resource_running():
         # will lose quorum, and with RA running
         # no reload, no restart cluster service
@@ -121,7 +128,7 @@ class QDevice(object):
     qdevice_db_path = "/etc/corosync/qdevice/net/nssdb"
 
     def __init__(self, qnetd_addr, port=5403, algo="ffsplit", tie_breaker="lowest",
-            tls="on", cluster_node=None, cmds=None, mode=None, cluster_name=None, is_stage=False):
+            tls="on", ssh_user=None, cluster_node=None, cmds=None, mode=None, cluster_name=None, is_stage=False):
         """
         Init function
         """
@@ -130,12 +137,14 @@ class QDevice(object):
         self.algo = algo
         self.tie_breaker = tie_breaker
         self.tls = tls
+        self.ssh_user = ssh_user
         self.cluster_node = cluster_node
         self.cmds = cmds
         self.mode = mode
         self.cluster_name = cluster_name
         self.qdevice_reload_policy = QdevicePolicy.QDEVICE_RESTART
         self.is_stage = is_stage
+        self.using_diskless_sbd = False
 
     @property
     def qnetd_cacert_on_qnetd(self):
@@ -200,52 +209,82 @@ class QDevice(object):
         """
         return "{}/{}/{}".format(self.qdevice_path, self.cluster_node, self.qdevice_p12_filename)
 
-    def valid_qdevice_options(self):
-        """
-        Validate qdevice related options
-        """
+    @staticmethod
+    def check_qnetd_addr(qnetd_addr):
         qnetd_ip = None
-
-        if not utils.package_is_installed("corosync-qdevice"):
-            raise ValueError("Package \"corosync-qdevice\" not installed on this node")
-
         try:
             # socket.getaddrinfo works for both ipv4 and ipv6 address
             # The function returns a list of 5-tuples with the following structure:
             # (family, type, proto, canonname, sockaddr)
             # sockaddr is a tuple describing a socket address, whose format depends on the returned family
             # (a (address, port) 2-tuple for AF_INET, a (address, port, flow info, scope id) 4-tuple for AF_INET6)
-            res = socket.getaddrinfo(self.qnetd_addr, None)
+            res = socket.getaddrinfo(qnetd_addr, None)
             qnetd_ip = res[0][-1][0]
         except socket.error:
-            raise ValueError("host \"{}\" is unreachable".format(self.qnetd_addr))
+            raise ValueError("host \"{}\" is unreachable".format(qnetd_addr))
 
-        utils.ping_node(self.qnetd_addr)
+        utils.ping_node(qnetd_addr)
 
         if utils.InterfacesInfo.ip_in_local(qnetd_ip):
             raise ValueError("host for qnetd must be a remote one")
 
         if not utils.check_port_open(qnetd_ip, 22):
-            raise ValueError("ssh service on \"{}\" not available".format(self.qnetd_addr))
+            raise ValueError("ssh service on \"{}\" not available".format(qnetd_addr))
 
-        if not utils.valid_port(self.port):
+    @staticmethod
+    def check_qdevice_port(qdevice_port):
+        if not utils.valid_port(qdevice_port):
             raise ValueError("invalid qdevice port range(1024 - 65535)")
 
-        if self.algo not in ("ffsplit", "lms"):
-            raise ValueError("invalid ALGORITHM choice: '{}' (choose from 'ffsplit', 'lms')".format(self.algo))
+    @staticmethod
+    def check_qdevice_algo(qdevice_algo):
+        if qdevice_algo not in ("ffsplit", "lms"):
+            raise ValueError("invalid ALGORITHM choice: '{}' (choose from 'ffsplit', 'lms')".format(qdevice_algo))
 
-        if self.tie_breaker not in ("lowest", "highest") and not utils.valid_nodeid(self.tie_breaker):
+    @staticmethod
+    def check_qdevice_tie_breaker(qdevice_tie_breaker):
+        if qdevice_tie_breaker not in ("lowest", "highest") and not utils.valid_nodeid(qdevice_tie_breaker):
             raise ValueError("invalid qdevice tie_breaker(lowest/highest/valid_node_id)")
 
-        if self.tls not in ("on", "off", "required"):
-            raise ValueError("invalid TLS choice: '{}' (choose from 'on', 'off', 'required')".format(self.tls))
+    @staticmethod
+    def check_qdevice_tls(qdevice_tls):
+        if qdevice_tls not in ("on", "off", "required"):
+            raise ValueError("invalid TLS choice: '{}' (choose from 'on', 'off', 'required')".format(qdevice_tls))
 
-        if self.cmds:
-            for cmd in self.cmds.strip(';').split(';'):
-                if not cmd.startswith('/'):
-                    raise ValueError("commands for heuristics should be absolute path")
-                if not os.path.exists(cmd.split()[0]):
-                    raise ValueError("command {} not exist".format(cmd.split()[0]))
+    @staticmethod
+    def check_qdevice_heuristics_mode(mode):
+        if not mode:
+            return
+        if mode not in ("on", "sync", "off"):
+            raise ValueError("invalid MODE choice: '{}' (choose from 'on', 'sync', 'off')".format(mode))
+
+    @staticmethod
+    def check_qdevice_heuristics(cmds):
+        if not cmds:
+            return
+        for cmd in cmds.strip(';').split(';'):
+            if not cmd.startswith('/'):
+                raise ValueError("commands for heuristics should be absolute path")
+            if not os.path.exists(cmd.split()[0]):
+                raise ValueError("command {} not exist".format(cmd.split()[0]))
+
+    @staticmethod
+    def check_package_installed(pkg, remote=None):
+        if not utils.package_is_installed(pkg, remote_addr=remote):
+            raise ValueError("Package \"{}\" not installed on {}".format(pkg, remote if remote else "this node"))
+
+    def valid_qdevice_options(self):
+        """
+        Validate qdevice related options
+        """
+        self.check_package_installed("corosync-qdevice")
+        self.check_qnetd_addr(self.qnetd_addr)
+        self.check_qdevice_port(self.port)
+        self.check_qdevice_algo(self.algo)
+        self.check_qdevice_tie_breaker(self.tie_breaker)
+        self.check_qdevice_tls(self.tls)
+        self.check_qdevice_heuristics(self.cmds)
+        self.check_qdevice_heuristics_mode(self.mode)
 
     def valid_qnetd(self):
         """
@@ -254,10 +293,10 @@ class QDevice(object):
         exception_msg = ""
         suggest = ""
         duplicated_cluster_name = False
-        if not utils.package_is_installed("corosync-qnetd", self.qnetd_addr):
+        if not utils.package_is_installed("corosync-qnetd", remote_addr=self.qnetd_addr):
             exception_msg = "Package \"corosync-qnetd\" not installed on {}!".format(self.qnetd_addr)
             suggest = "install \"corosync-qnetd\" on {}".format(self.qnetd_addr)
-        elif utils.service_is_active("corosync-qnetd", self.qnetd_addr):
+        elif utils.service_is_active("corosync-qnetd", remote_addr=self.qnetd_addr):
             cmd = "corosync-qnetd-tool -l -c {}".format(self.cluster_name)
             if utils.get_stdout_or_raise_error(cmd, remote=self.qnetd_addr):
                 duplicated_cluster_name = True
@@ -317,7 +356,7 @@ class QDevice(object):
 
         cmd = "corosync-qnetd-certutil -i"
         desc = "Step 1: Initialize database on {}".format(self.qnetd_addr)
-        logger_utils.log_only_to_file(desc)
+        QDevice.log_only_to_file(desc, cmd)
         parallax.parallax_call([self.qnetd_addr], cmd)
 
     def fetch_qnetd_crt_from_qnetd(self):
@@ -330,8 +369,8 @@ class QDevice(object):
             return
 
         desc = "Step 2: Fetch {} from {}".format(self.qnetd_cacert_filename, self.qnetd_addr)
-        logger_utils.log_only_to_file(desc)
-        parallax.parallax_slurp([self.qnetd_addr], self.qdevice_path, self.qnetd_cacert_on_qnetd)
+        QDevice.log_only_to_file(desc)
+        self._fetch_file_to_local(self.qnetd_addr, self.qnetd_cacert_on_qnetd, '{}/{}'.format(self.qdevice_path, self.qnetd_addr))
 
     def copy_qnetd_crt_to_cluster(self):
         """
@@ -344,11 +383,43 @@ class QDevice(object):
             return
 
         desc = "Step 3: Copy exported {} to {}".format(self.qnetd_cacert_filename, node_list)
-        logger_utils.log_only_to_file(desc)
-        parallax.parallax_copy(
-                node_list,
-                os.path.dirname(self.qnetd_cacert_on_local),
-                self.qdevice_path)
+        QDevice.log_only_to_file(desc)
+        self._copy_file_to_remote_hosts(os.path.dirname(self.qnetd_cacert_on_local), node_list, self.qdevice_path)
+
+    @classmethod
+    def _copy_file_to_remote_host(cls, local_file, remote_host: str, remote_path):
+        local_user, remote_user = utils.user_pair_for_ssh(remote_host)
+        with tempfile.NamedTemporaryFile('w', encoding='utf-8') as tmp:
+            tmp.write("put -pr '{}' '{}'\n".format(local_file, remote_path))
+            tmp.flush()
+            # we can not su to a non-root user, since reading the source file may need privilege.
+            cmd = "sftp {} -o IdentityFile=~{}/.ssh/id_rsa -o BatchMode=yes -s 'sudo PATH=/usr/lib/ssh:/usr/libexec/ssh /bin/sh -c \"exec sftp-server\"' -b {} {}@{}".format(
+                constants.SSH_OPTION,
+                local_user,
+                # FIXME: sftp-server is not always in /usr/lib/ssh
+                tmp.name,
+                remote_user, cls._enclose_inet6_addr(remote_host),
+            )
+            result = subprocess.run(
+                ['/bin/sh', '-c', cmd],
+                stdin=subprocess.DEVNULL,
+                stdout=subprocess.PIPE,
+                stderr=subprocess.STDOUT,
+            )
+        if result.returncode != 0:
+            utils.fatal('Failed to copy file from {} to {}:{}: {}: {}'.format(local_file, remote_host, remote_path, cmd, utils.to_ascii(result.stdout)))
+
+    @staticmethod
+    def _enclose_inet6_addr(addr: str):
+        if ':' in addr:
+            return f'[{addr}]'
+        else:
+            return addr
+
+    @classmethod
+    def _copy_file_to_remote_hosts(cls, local_file, remote_hosts: typing.Iterable[str], remote_path):
+        for host in remote_hosts:
+            cls._copy_file_to_remote_host(local_file, host, remote_path)
 
     def init_db_on_cluster(self):
         """
@@ -360,7 +431,7 @@ class QDevice(object):
         node_list = utils.list_cluster_nodes()
         cmd = "corosync-qdevice-net-certutil -i -c {}".format(self.qnetd_cacert_on_local)
         desc = "Step 4: Initialize database on {}".format(node_list)
-        logger_utils.log_only_to_file(desc)
+        QDevice.log_only_to_file(desc, cmd)
         parallax.parallax_call(node_list, cmd)
 
     def create_ca_request(self):
@@ -371,8 +442,8 @@ class QDevice(object):
         /usr/sbin/corosync-qdevice-net-certutil -r -n Cluster
         (Cluster name must match cluster_name key in the corosync.conf)
         """
-        logger_utils.log_only_to_file("Step 5: Generate certificate request {}".format(self.qdevice_crq_filename))
         cmd = "corosync-qdevice-net-certutil -r -n {}".format(self.cluster_name)
+        QDevice.log_only_to_file("Step 5: Generate certificate request {}".format(self.qdevice_crq_filename), cmd)
         utils.get_stdout_or_raise_error(cmd)
 
     def copy_crq_to_qnetd(self):
@@ -382,11 +453,8 @@ class QDevice(object):
         Copy exported CRQ to QNetd server
         """
         desc = "Step 6: Copy {} to {}".format(self.qdevice_crq_filename, self.qnetd_addr)
-        logger_utils.log_only_to_file(desc)
-        parallax.parallax_copy(
-                [self.qnetd_addr],
-                self.qdevice_crq_on_local,
-                self.qdevice_crq_on_qnetd)
+        QDevice.log_only_to_file(desc)
+        self._copy_file_to_remote_host(self.qdevice_crq_on_local, self.qnetd_addr, self.qdevice_crq_on_qnetd)
 
     def sign_crq_on_qnetd(self):
         """
@@ -396,9 +464,9 @@ class QDevice(object):
         corosync-qnetd-certutil -s -c qdevice-net-node.crq -n Cluster
         """
         desc = "Step 7: Sign and export cluster certificate on {}".format(self.qnetd_addr)
-        logger_utils.log_only_to_file(desc)
         cmd = "corosync-qnetd-certutil -s -c {} -n {}".\
                 format(self.qdevice_crq_on_qnetd, self.cluster_name)
+        QDevice.log_only_to_file(desc, cmd)
         parallax.parallax_call([self.qnetd_addr], cmd)
 
     def fetch_cluster_crt_from_qnetd(self):
@@ -408,11 +476,22 @@ class QDevice(object):
         Copy exported CRT to node where certificate request was created
         """
         desc = "Step 8: Fetch {} from {}".format(os.path.basename(self.qnetd_cluster_crt_on_qnetd), self.qnetd_addr)
-        logger_utils.log_only_to_file(desc)
-        parallax.parallax_slurp(
-                [self.qnetd_addr],
-                self.qdevice_path,
-                self.qnetd_cluster_crt_on_qnetd)
+        QDevice.log_only_to_file(desc)
+        self._fetch_file_to_local(self.qnetd_addr, self.qnetd_cluster_crt_on_qnetd, '{}/{}'.format(self.qdevice_path, self.qnetd_addr))
+
+    @classmethod
+    def _fetch_file_to_local(cls, remote_host: str, remote_path: str, local_dir: str):
+        basename = os.path.basename(remote_path)
+        rc, stdout, stderr = utils.get_stdout_stderr_auto_ssh_no_input(
+            remote_host,
+            "cat '{}'".format(remote_path),
+            raw=True,
+        )
+        if rc != 0:
+            utils.fatal("Failed to fetch file {} from {}: {}".format(remote_path, remote_host, stderr))
+        os.makedirs(local_dir, exist_ok=True)
+        with open('{}/{}'.format(local_dir, basename), 'wb') as f:
+            f.write(stdout)
 
     def import_cluster_crt(self):
         """
@@ -421,8 +500,10 @@ class QDevice(object):
         Import certificate on node where certificate request was created by
         running /usr/sbin/corosync-qdevice-net-certutil -M -c cluster-Cluster.crt
         """
-        logger_utils.log_only_to_file("Step 9: Import certificate file {} on local".format(os.path.basename(self.qnetd_cluster_crt_on_local)))
         cmd = "corosync-qdevice-net-certutil -M -c {}".format(self.qnetd_cluster_crt_on_local)
+        QDevice.log_only_to_file(
+                "Step 9: Import certificate file {} on local".format(os.path.basename(self.qnetd_cluster_crt_on_local)),
+                cmd)
         utils.get_stdout_or_raise_error(cmd)
 
     def copy_p12_to_cluster(self):
@@ -436,11 +517,8 @@ class QDevice(object):
             return
 
         desc = "Step 10: Copy {} to {}".format(self.qdevice_p12_filename, node_list)
-        logger_utils.log_only_to_file(desc)
-        parallax.parallax_copy(
-                node_list,
-                self.qdevice_p12_on_local,
-                os.path.dirname(self.qdevice_p12_on_local))
+        QDevice.log_only_to_file(desc)
+        self._copy_file_to_remote_hosts(self.qdevice_p12_on_local, node_list, self.qdevice_p12_on_local)
 
     def import_p12_on_cluster(self):
         """
@@ -454,8 +532,8 @@ class QDevice(object):
             return
 
         desc = "Step 11: Import {} on {}".format(self.qdevice_p12_filename, node_list)
-        logger_utils.log_only_to_file(desc)
         cmd = "corosync-qdevice-net-certutil -m -c {}".format(self.qdevice_p12_on_local)
+        QDevice.log_only_to_file(desc, cmd)
         parallax.parallax_call(node_list, cmd)
 
     def certificate_process_on_init(self):
@@ -484,11 +562,8 @@ class QDevice(object):
             return
 
         desc = "Step 1: Fetch {} from {}".format(self.qnetd_cacert_filename, self.cluster_node)
-        logger_utils.log_only_to_file(desc)
-        parallax.parallax_slurp(
-                [self.cluster_node],
-                self.qdevice_path,
-                self.qnetd_cacert_on_local)
+        QDevice.log_only_to_file(desc)
+        self._fetch_file_to_local(self.cluster_node, self.qnetd_cacert_on_local, '{}/{}'.format(self.qdevice_path, self.cluster_node))
 
     def init_db_on_local(self):
         """
@@ -500,8 +575,8 @@ class QDevice(object):
         if os.path.exists(self.qdevice_db_path):
             utils.rmdir_r(self.qdevice_db_path)
 
-        logger_utils.log_only_to_file("Step 2: Initialize database on local")
         cmd = "corosync-qdevice-net-certutil -i -c {}".format(self.qnetd_cacert_on_cluster)
+        QDevice.log_only_to_file("Step 2: Initialize database on local", cmd)
         utils.get_stdout_or_raise_error(cmd)
 
     def fetch_p12_from_cluster(self):
@@ -514,11 +589,8 @@ class QDevice(object):
             return
 
         desc = "Step 3: Fetch {} from {}".format(self.qdevice_p12_filename, self.cluster_node)
-        logger_utils.log_only_to_file(desc)
-        parallax.parallax_slurp(
-                [self.cluster_node],
-                self.qdevice_path,
-                self.qdevice_p12_on_local)
+        QDevice.log_only_to_file(desc)
+        self._fetch_file_to_local(self.cluster_node, self.qdevice_p12_on_local, '{}/{}'.format(self.qdevice_path, self.cluster_node))
 
     def import_p12_on_local(self):
         """
@@ -526,8 +598,8 @@ class QDevice(object):
         Step 4
         Import cluster certificate and key
         """
-        logger_utils.log_only_to_file("Step 4: Import cluster certificate and key")
         cmd = "corosync-qdevice-net-certutil -m -c {}".format(self.qdevice_p12_on_cluster)
+        QDevice.log_only_to_file("Step 4: Import cluster certificate and key", cmd)
         utils.get_stdout_or_raise_error(cmd)
 
     def certificate_process_on_join(self):
@@ -581,6 +653,7 @@ class QDevice(object):
             return
         node_list = addr_list if addr_list else utils.list_cluster_nodes()
         cmd = "rm -rf {}/*".format(QDevice.qdevice_path)
+        QDevice.log_only_to_file("Remove qdevice database", cmd)
         parallax.parallax_call(node_list, cmd)
 
     @classmethod
@@ -635,18 +708,16 @@ class QDevice(object):
         """
         Adjust SBD_WATCHDOG_TIMEOUT when configuring qdevice and diskless SBD
         """
-        if self.is_stage:
-            from .sbd import SBDManager, SBDTimeout
-            utils.check_all_nodes_reachable()
-            using_diskless_sbd = SBDManager.is_using_diskless_sbd()
-            self.qdevice_reload_policy = evaluate_qdevice_quorum_effect(QDEVICE_ADD, using_diskless_sbd)
-            # add qdevice after diskless sbd started
-            if using_diskless_sbd:
-                res = SBDManager.get_sbd_value_from_config("SBD_WATCHDOG_TIMEOUT")
-                if not res or int(res) < SBDTimeout.SBD_WATCHDOG_TIMEOUT_DEFAULT_WITH_QDEVICE:
-                    sbd_watchdog_timeout_qdevice = SBDTimeout.SBD_WATCHDOG_TIMEOUT_DEFAULT_WITH_QDEVICE
-                    SBDManager.update_configuration({"SBD_WATCHDOG_TIMEOUT": str(sbd_watchdog_timeout_qdevice)})
-                    utils.set_property(stonith_timeout=SBDTimeout.get_stonith_timeout())
+        from .sbd import SBDManager, SBDTimeout
+        utils.check_all_nodes_reachable()
+        self.using_diskless_sbd = SBDManager.is_using_diskless_sbd()
+        # add qdevice after diskless sbd started
+        if self.using_diskless_sbd:
+            res = SBDManager.get_sbd_value_from_config("SBD_WATCHDOG_TIMEOUT")
+            if not res or int(res) < SBDTimeout.SBD_WATCHDOG_TIMEOUT_DEFAULT_WITH_QDEVICE:
+                sbd_watchdog_timeout_qdevice = SBDTimeout.SBD_WATCHDOG_TIMEOUT_DEFAULT_WITH_QDEVICE
+                SBDManager.update_configuration({"SBD_WATCHDOG_TIMEOUT": str(sbd_watchdog_timeout_qdevice)})
+                utils.set_property("stonith-timeout", SBDTimeout.get_stonith_timeout())
 
     @qnetd_lock_for_same_cluster_name
     def config_and_start_qdevice(self):
@@ -658,6 +729,7 @@ class QDevice(object):
             with logger_utils.status_long("Qdevice certification process"):
                 self.certificate_process_on_init()
         self.adjust_sbd_watchdog_timeout_with_qdevice()
+        self.qdevice_reload_policy = evaluate_qdevice_quorum_effect(QDEVICE_ADD, self.using_diskless_sbd, self.is_stage)
         self.config_qdevice()
         self.start_qdevice_service()
 
@@ -671,3 +743,9 @@ class QDevice(object):
         if res:
             qnetd_host = corosync.get_value('quorum.device.net.host')
             logger.warning("Qdevice's vote is 0, which simply means Qdevice can't talk to Qnetd({}) for various reasons.".format(qnetd_host))
+
+    @staticmethod
+    def log_only_to_file(desc, cmd=None):
+        logger_utils.log_only_to_file(desc)
+        if cmd:
+            logger_utils.log_only_to_file(f"Run: {cmd}")
