@@ -11,7 +11,7 @@ import datetime
 import shutil
 
 from crmsh import utils as crmutils
-from crmsh import config, log
+from crmsh import config, log, userdir
 from crmsh.report import constants, utillib
 
 
@@ -24,7 +24,7 @@ def collect_for_nodes(nodes, arg_str):
     """
     process_list = []
     for node in nodes.split():
-        if utillib.node_needs_pwd(node):
+        if node in constants.SSH_PASSWORD_NODES:
             logger.info("Please provide password for %s at %s", utillib.say_ssh_user(), node)
             logger.info("Note that collecting data will take a while.")
             utillib.start_slave_collector(node, arg_str)
@@ -52,7 +52,7 @@ def dump_env():
     env_dict["SKIP_LVL"] = constants.SKIP_LVL
     env_dict["EXTRA_LOGS"] = constants.EXTRA_LOGS
     env_dict["PCMK_LOG"] = constants.PCMK_LOG
-    env_dict["VERBOSITY"] = int(constants.VERBOSITY)
+    env_dict["VERBOSITY"] = int(config.report.verbosity) or (1 if config.core.debug else 0)
 
     res_str = ""
     for k, v in env_dict.items():
@@ -133,8 +133,7 @@ def load_env(env_str):
     constants.SKIP_LVL = utillib.str_to_bool(env_dict["SKIP_LVL"])
     constants.EXTRA_LOGS = env_dict["EXTRA_LOGS"]
     constants.PCMK_LOG = env_dict["PCMK_LOG"]
-    constants.VERBOSITY = int(env_dict["VERBOSITY"])
-    config.report.verbosity = constants.VERBOSITY
+    config.report.verbosity = env_dict["VERBOSITY"]
 
 
 def parse_argument(argv):
@@ -151,6 +150,7 @@ def parse_argument(argv):
     else:
         usage("short")
 
+    verbosity = 0
     for args, option in opt:
         if args == '-h':
             usage()
@@ -191,11 +191,12 @@ def parse_argument(argv):
         if args == "-E":
             constants.EXTRA_LOGS += " %s" % option
         if args == "-v":
-            constants.VERBOSITY += 1
-            config.report.verbosity = constants.VERBOSITY
+            verbosity += 1
         if args == '-d':
             constants.COMPRESS = False
 
+    config.report.verbosity = verbosity
+
     if config.report.sanitize_rule:
         constants.DO_SANITIZE = True
         temp_pattern_set = set()
@@ -266,16 +267,6 @@ def run():
             # if the ssh user was supplied, consider that it
             # works; helps reduce the number of ssh invocations
             utillib.find_ssh_user()
-            if constants.SSH_USER:
-                constants.SSH_OPTS += " -o User=%s" % constants.SSH_USER
-        # assume that only root can collect data
-        if ((not constants.SSH_USER) and (os.getuid() not in [0, 90])) or \
-           constants.SSH_USER and constants.SSH_USER not in ["root", "hacluster"]:
-            logger.debug("ssh user other than root, use sudo")
-            constants.SUDO = "sudo -u root"
-        if os.getuid() != 0:
-            logger.debug("local user other than root, use sudo")
-            constants.LOCAL_SUDO = "sudo -u root"
 
     #
     # find the logs and cut out the segment for the period
