@@ -23,7 +23,7 @@ from inspect import getmembers, isfunction
 
 import crmsh.config
 from crmsh import utils as crmutils
-from crmsh import corosync, log
+from crmsh import corosync, log, userdir
 from crmsh.report import constants, collect
 
 
@@ -321,7 +321,7 @@ def collect_info():
             if dump_logset(l, constants.FROM_TIME, constants.TO_TIME, outf):
                 log_size(l, outf+'.info')
         else:
-            logger.warning("could not figure out the log format of %s", l)
+            logger.debug("could not figure out the log format of %s", l)
 
 
 def collect_journal(from_t, to_t, outf):
@@ -342,7 +342,7 @@ def collect_journal(from_t, to_t, outf):
 
     logger.debug("journalctl from: '%d' until: '%d' from_time: '%s' to_time: '%s' > %s",
               from_t, to_t, from_time, to_time, outf)
-    cmd = 'journalctl -o short-iso --since "%s" --until "%s" --no-pager | tail -n +2' % \
+    cmd = 'journalctl -o short-iso-precise --since "%s" --until "%s" --no-pager | tail -n +2' % \
           (from_time, to_time)
     crmutils.str2file(get_command_info(cmd)[1], outf)
 
@@ -623,39 +623,37 @@ def find_log():
 
 
 def find_ssh_user():
-    ssh_user = "__undef"
-    if not constants.SSH_USER:
-        try_user_list = "__default " + constants.TRY_SSH
-    else:
-        try_user_list = constants.SSH_USER
+    ssh_user = ""
+    ssh_user_try_list = []
+    if constants.SSH_USER:
+        ssh_user_try_list.append(constants.SSH_USER)
+    sudoer = userdir.get_sudoer()
+    if sudoer:
+        ssh_user_try_list.append(sudoer)
+    current_user = userdir.getuser()
+    ssh_user_try_list.append(current_user)
 
     for n in constants.NODES.split():
-        rc = 1
         if n == constants.WE:
             continue
-        for u in try_user_list.split():
-            if u != '__default':
-                ssh_s = '@'.join((u, n))
-            else:
-                ssh_s = n
-
-            if test_ssh_conn(ssh_s):
+        rc = False
+        for u in ssh_user_try_list:
+            ssh_s = f"{u}@{n}"
+            if not crmutils.check_ssh_passwd_need(u, u, n):
                 logger.debug("ssh %s OK", ssh_s)
                 ssh_user = u
-                try_user_list = u
-                rc = 0
+                rc = True
                 break
             else:
                 logger.debug("ssh %s failed", ssh_s)
-        if rc == 1:
-            constants.SSH_PASSWORD_NODES += " %s" % n
+        if not rc:
+            constants.SSH_PASSWORD_NODES.append(n)
 
     if constants.SSH_PASSWORD_NODES:
         logger.warning("passwordless ssh to node(s) %s does not work", constants.SSH_PASSWORD_NODES)
-    if ssh_user == "__undef":
-        return
-    if ssh_user != "__default":
-        constants.SSH_USER = ssh_user
+    if ssh_user:
+        constants.SUDO = "" if ssh_user in ["root", "hacluster"] else "sudo"
+    constants.SSH_USER = ssh_user if ssh_user else ""
 
 
 def findln_by_time(data, ts):
@@ -1119,13 +1117,6 @@ def mktemplate(argv):
     crmutils.str2file(out_string, os.path.join(workdir, constants.DESCRIPTION_F))
 
 
-def node_needs_pwd(node):
-    for n in constants.SSH_PASSWORD_NODES.split():
-        if n == node:
-            return True
-    return False
-
-
 def pe_to_dot(pe_file):
     dotf = '.'.join(pe_file.split('.')[:-1]) + '.dot'
     cmd = "%s -D %s -x %s" % (constants.PTEST, dotf, pe_file)
@@ -1367,10 +1358,11 @@ def start_slave_collector(node, arg_str):
             cmd += " {}".format(str(item))
         _, out = crmutils.get_stdout(cmd)
     else:
+        node = f"{constants.SSH_USER}@{node}" if constants.SSH_USER else node
         cmd = r'ssh {} {} "{} {}"'.format(constants.SSH_OPTS, node, constants.SUDO, cmd)
         for item in arg_str.split():
             cmd += " {}".format(str(item))
-        code, out, err = crmutils.get_stdout_stderr(cmd)
+        code, out, err = crmutils.su_get_stdout_stderr(constants.SSH_USER, cmd)
         if code != 0:
             logger.warning(err)
             for ip in get_peer_ip():
@@ -1383,6 +1375,9 @@ def start_slave_collector(node, arg_str):
         if err:
             print(err)
 
+    if out == '': # if we couldn't get anything
+        return
+
     compress_data = ""
     for data in out.split('\n'):
         if data.startswith(constants.COMPRESS_DATA_FLAG):
@@ -1404,15 +1399,6 @@ def tail(n, indata):
     return indata.split('\n')[-n:]
 
 
-def test_ssh_conn(addr):
-    cmd = r"ssh %s -T -o Batchmode=yes %s true" % (constants.SSH_OPTS, addr)
-    code, _ = get_command_info(cmd)
-    if code == 0:
-        return True
-    else:
-        return False
-
-
 def dump_D_process():
     '''
     dump D-state process stack
@@ -1541,7 +1527,11 @@ def read_from_file(infile):
     data = None
     _open = get_open_method(infile)
     with _open(infile, 'rt', encoding='utf-8', errors='replace') as f:
-        data = f.read()
+        try:
+            data = f.read()
+        except Exception as err:
+            logger.error("When reading file \"%s\": %s", infile, str(err))
+            return None
     return crmutils.to_ascii(data)
 
 
