@@ -24,7 +24,7 @@ class SBDTimeout(object):
     SBD_WATCHDOG_TIMEOUT_DEFAULT_WITH_QDEVICE = 35
     QDEVICE_SYNC_TIMEOUT_MARGIN = 5
 
-    def __init__(self, context=None, removing=False):
+    def __init__(self, context=None):
         """
         Init function
         """
@@ -34,10 +34,16 @@ class SBDTimeout(object):
         self.sbd_watchdog_timeout = self.SBD_WATCHDOG_TIMEOUT_DEFAULT
         self.stonith_watchdog_timeout = self.STONITH_WATCHDOG_TIMEOUT_DEFAULT
         self.sbd_delay_start = None
-        self.removing = removing
         self.two_node_without_qdevice = False
 
-    def set_sbd_watchdog_timeout(self):
+    def initialize_timeout(self):
+        self._set_sbd_watchdog_timeout()
+        if self.context.diskless_sbd:
+            self._adjust_sbd_watchdog_timeout_with_diskless_and_qdevice()
+        else:
+            self._set_sbd_msgwait()
+
+    def _set_sbd_watchdog_timeout(self):
         """
         Set sbd_watchdog_timeout from profiles.yml if exists
         Then adjust it if in s390 environment
@@ -48,7 +54,7 @@ class SBDTimeout(object):
             logger.warning("sbd_watchdog_timeout is set to %d for s390, it was %d", self.SBD_WATCHDOG_TIMEOUT_DEFAULT_S390, self.sbd_watchdog_timeout)
             self.sbd_watchdog_timeout = self.SBD_WATCHDOG_TIMEOUT_DEFAULT_S390
 
-    def set_sbd_msgwait(self):
+    def _set_sbd_msgwait(self):
         """
         Set sbd msgwait from profiles.yml if exists
         Default is 2 * sbd_watchdog_timeout
@@ -62,7 +68,7 @@ class SBDTimeout(object):
                 sbd_msgwait = sbd_msgwait_default
         self.sbd_msgwait = sbd_msgwait
 
-    def adjust_sbd_watchdog_timeout_with_diskless_and_qdevice(self):
+    def _adjust_sbd_watchdog_timeout_with_diskless_and_qdevice(self):
         """
         When using diskless SBD with Qdevice, adjust value of sbd_watchdog_timeout
         """
@@ -116,7 +122,7 @@ class SBDTimeout(object):
         """
         Load necessary configurations for both disk-based/disk-less sbd
         """
-        self.two_node_without_qdevice = utils.is_2node_cluster_without_qdevice(self.removing)
+        self.two_node_without_qdevice = utils.is_2node_cluster_without_qdevice()
 
         dev_list = SBDManager.get_sbd_device_from_config()
         if dev_list:  # disk-based
@@ -206,29 +212,14 @@ class SBDTimeout(object):
         utils.mkdirp(SBD_SYSTEMD_DELAY_START_DIR)
         sbd_delay_start_file = "{}/sbd_delay_start.conf".format(SBD_SYSTEMD_DELAY_START_DIR)
         utils.str2file("[Service]\nTimeoutSec={}".format(int(1.2*int(sbd_delay_start_value))), sbd_delay_start_file)
-        bootstrap.csync2_update(SBD_SYSTEMD_DELAY_START_DIR)
+        bootstrap.sync_file(SBD_SYSTEMD_DELAY_START_DIR)
         utils.cluster_run_cmd("systemctl daemon-reload")
 
     def adjust_stonith_timeout(self):
         """
         Adjust stonith-timeout property
         """
-        utils.set_property_conditionally("stonith-timeout", self.get_stonith_timeout_expected())
-
-    def adjust_pcmk_delay_max(self):
-        """
-        Adjust pcmk_delay_max parameter for sbd ra
-        """
-        # TODO this function should be outside of sbd.py, to adjust any fence device
-
-        if not xmlutil.CrmMonXmlParser.is_resource_configured(SBDManager.SBD_RA):
-            return
-
-        if self.two_node_without_qdevice:
-            cmd = "crm resource param {} set pcmk_delay_max {}s".format(SBDManager.SBD_RA_ID, self.pcmk_delay_max)
-        else:
-            cmd = "crm resource param {} delete pcmk_delay_max".format(SBDManager.SBD_RA_ID)
-        utils.get_stdout_or_raise_error(cmd)
+        utils.set_property("stonith-timeout", self.get_stonith_timeout_expected(), conditional=True)
 
     def adjust_sbd_delay_start(self):
         """
@@ -244,17 +235,16 @@ class SBDTimeout(object):
             SBDManager.update_configuration({"SBD_DELAY_START": expected_value})
 
     @classmethod
-    def adjust_sbd_timeout_related_cluster_configuration(cls, removing=False):
+    def adjust_sbd_timeout_related_cluster_configuration(cls):
         """
         Adjust sbd timeout related configurations
         """
-        cls_inst = cls(removing=removing)
+        cls_inst = cls()
         cls_inst._load_configurations()
 
-        message = "Adjusting sbd related timeout values for 2-node cluster"
+        message = "Adjusting sbd related timeout values"
         with logger_utils.status_long(message):
             cls_inst.adjust_sbd_delay_start()
-            cls_inst.adjust_pcmk_delay_max()
             cls_inst.adjust_stonith_timeout()
             cls_inst.adjust_systemd_start_timeout()
 
@@ -294,7 +284,8 @@ class SBDManager(object):
         self._context = context
         self._delay_start = False
         self.timeout_inst = None
-        self.no_overwrite = False
+        self.no_overwrite_map = {}
+        self.no_update_config = False
 
     @staticmethod
     def _get_device_uuid(dev, node=None):
@@ -330,6 +321,12 @@ class SBDManager(object):
                 raise ValueError("{} doesn't look like a block device".format(dev))
             self._compare_device_uuid(dev, compare_node_list)
 
+    def _no_overwrite_check(self, dev):
+        """
+        Check if device already initialized and if need to overwrite
+        """
+        return SBDManager.has_sbd_device_already_initialized(dev) and not bootstrap.confirm("SBD is already configured to use {} - overwrite?".format(dev))
+
     def _get_sbd_device_interactive(self):
         """
         Get sbd device on interactive mode
@@ -345,27 +342,35 @@ class SBDManager(object):
             return
 
         configured_dev_list = self._get_sbd_device_from_config()
-        if configured_dev_list and not bootstrap.confirm("SBD is already configured to use {} - overwrite?".format(';'.join(configured_dev_list))):
-            self.no_overwrite = True
+        for dev in configured_dev_list:
+            self.no_overwrite_map[dev] = self._no_overwrite_check(dev)
+        if self.no_overwrite_map and all(self.no_overwrite_map.values()):
+            self.no_update_config = True
             return configured_dev_list
 
         dev_list = []
         dev_looks_sane = False
         while not dev_looks_sane:
             dev = bootstrap.prompt_for_string('Path to storage device (e.g. /dev/disk/by-id/...), or "none" for diskless sbd, use ";" as separator for multi path', r'none|\/.*')
-            if not dev:
-                continue
             if dev == "none":
                 self.diskless_sbd = True
                 return
+
             dev_list = utils.re_split_string(self.PARSE_RE, dev)
             try:
                 self._verify_sbd_device(dev_list)
             except ValueError as err_msg:
                 logger.error(str(err_msg))
                 continue
-            for dev_item in dev_list:
-                logger.warning("All data on {} will be destroyed!".format(dev_item))
+
+            for dev in dev_list:
+                if dev not in self.no_overwrite_map:
+                    self.no_overwrite_map[dev] = self._no_overwrite_check(dev)
+                if self.no_overwrite_map[dev]:
+                    if dev == dev_list[-1]:
+                        return dev_list
+                    continue
+                logger.warning("All data on {} will be destroyed!".format(dev))
                 if bootstrap.confirm('Are you sure you wish to use this device?'):
                     dev_looks_sane = True
                 else:
@@ -382,6 +387,10 @@ class SBDManager(object):
         if self.sbd_devices_input:
             dev_list = utils.parse_append_action_argument(self.sbd_devices_input)
             self._verify_sbd_device(dev_list)
+            for dev in dev_list:
+                self.no_overwrite_map[dev] = self._no_overwrite_check(dev)
+            if all(self.no_overwrite_map.values()) and dev_list == self._get_sbd_device_from_config():
+                self.no_update_config = True
         elif not self.diskless_sbd:
             dev_list = self._get_sbd_device_interactive()
         self._sbd_devices = dev_list
@@ -396,21 +405,19 @@ class SBDManager(object):
         msg = ""
         if self.diskless_sbd:
             msg = "Configuring diskless SBD"
-        elif not self.no_overwrite:
+        elif not all(self.no_overwrite_map.values()):
             msg = "Initializing SBD"
         if msg:
             logger.info(msg)
         self.timeout_inst = SBDTimeout(self._context)
-        self.timeout_inst.set_sbd_watchdog_timeout()
+        self.timeout_inst.initialize_timeout()
         if self.diskless_sbd:
-            self.timeout_inst.adjust_sbd_watchdog_timeout_with_diskless_and_qdevice()
             return
 
-        self.timeout_inst.set_sbd_msgwait()
         opt = "-4 {} -1 {}".format(self.timeout_inst.sbd_msgwait, self.timeout_inst.sbd_watchdog_timeout)
 
         for dev in self._sbd_devices:
-            if self.no_overwrite and SBDManager.has_sbd_device_already_initialized(dev):
+            if dev in self.no_overwrite_map and self.no_overwrite_map[dev]:
                 continue
             rc, _, err = bootstrap.invoke("sbd {} -d {} create".format(opt, dev))
             if not rc:
@@ -420,11 +427,11 @@ class SBDManager(object):
         """
         Update /etc/sysconfig/sbd
         """
-        if self.no_overwrite:
-            bootstrap.csync2_update(SYSCONFIG_SBD)
+        if self.no_update_config:
+            bootstrap.sync_file(SYSCONFIG_SBD)
             return
 
-        shutil.copyfile(self.SYSCONFIG_SBD_TEMPLATE, SYSCONFIG_SBD)
+        utils.copy_local_file(self.SYSCONFIG_SBD_TEMPLATE, SYSCONFIG_SBD)
         sbd_config_dict = {
                 "SBD_WATCHDOG_DEV": self._watchdog_inst.watchdog_device_name,
                 "SBD_WATCHDOG_TIMEOUT": str(self.timeout_inst.sbd_watchdog_timeout)
@@ -432,7 +439,7 @@ class SBDManager(object):
         if self._sbd_devices:
             sbd_config_dict["SBD_DEVICE"] = ';'.join(self._sbd_devices)
         utils.sysconfig_set(SYSCONFIG_SBD, **sbd_config_dict)
-        bootstrap.csync2_update(SYSCONFIG_SBD)
+        bootstrap.sync_file(SYSCONFIG_SBD)
 
     def _get_sbd_device_from_config(self):
         """
@@ -521,17 +528,20 @@ class SBDManager(object):
         # disk-based sbd
         if self._get_sbd_device_from_config():
             utils.get_stdout_or_raise_error("crm configure primitive {} {}".format(self.SBD_RA_ID, self.SBD_RA))
-            utils.set_property(stonith_enabled="true")
+            utils.set_property("stonith-enabled", "true")
         # disk-less sbd
         else:
+            if self.timeout_inst is None:
+                self.timeout_inst = SBDTimeout(self._context)
+                self.timeout_inst.initialize_timeout()
             cmd = self.DISKLESS_CRM_CMD.format(self.timeout_inst.stonith_watchdog_timeout, constants.STONITH_TIMEOUT_DEFAULT)
             utils.get_stdout_or_raise_error(cmd)
 
         # in sbd stage
         if self._context.cluster_is_running:
-            SBDTimeout.adjust_sbd_timeout_related_cluster_configuration()
+            bootstrap.adjust_properties()
 
-    def join_sbd(self, peer_host):
+    def join_sbd(self, remote_user, peer_host):
         """
         Function join_sbd running on join process only
         On joining process, check whether peer node has enabled sbd.service
@@ -544,7 +554,7 @@ class SBDManager(object):
         if not os.path.exists(SYSCONFIG_SBD) or not utils.service_is_enabled("sbd.service", peer_host):
             bootstrap.invoke("systemctl disable sbd.service")
             return
-        self._watchdog_inst = Watchdog(peer_host=peer_host)
+        self._watchdog_inst = Watchdog(remote_user=remote_user, peer_host=peer_host)
         self._watchdog_inst.join_watchdog()
         dev_list = self._get_sbd_device_from_config()
         if dev_list:
@@ -591,7 +601,7 @@ class SBDManager(object):
         Update and sync sbd configuration
         """
         utils.sysconfig_set(SYSCONFIG_SBD, **sbd_config_dict)
-        bootstrap.csync2_update(SYSCONFIG_SBD)
+        bootstrap.sync_file(SYSCONFIG_SBD)
 
     @staticmethod
     def get_sbd_value_from_config(key):
