@@ -5,6 +5,8 @@
 import sys
 import re
 from argparse import ArgumentParser, RawDescriptionHelpFormatter
+
+import crmsh.parallax
 from . import command
 from . import utils
 from . import scripts
@@ -13,6 +15,7 @@ from . import bootstrap
 from . import corosync
 from . import qdevice
 from .cibconfig import cib_factory
+from .ui_node import parse_option_for_nodes
 from . import constants
 
 
@@ -20,11 +23,6 @@ from . import log
 logger = log.setup_logger(__name__)
 
 
-class ArgParser(ArgumentParser):
-    def format_epilog(self, formatter):
-        return self.epilog or ""
-
-
 def parse_options(parser, args):
     try:
         options, args = parser.parse_known_args(list(args))
@@ -37,60 +35,6 @@ def parse_options(parser, args):
     return options, args
 
 
-def parse_option_for_nodes(context, *args):
-    """
-    Parse option for nodes
-    Return a node list
-    """
-    action_type = context.get_command_name()
-    action_target = "node" if action_type in ["standby", "online"] else "cluster service"
-    action = "{} {}".format(action_type, action_target)
-    usage_template = """
-Specify node(s) on which to {action}.
-If no nodes are specified, {action} on the local node.
-If --all is specified, {action} on all nodes."""
-    addtion_usage = ""
-    if action_type == "standby":
-        usage_template += """
-\n\nAdditionally, you may specify a lifetime for the standby---if set to
-"reboot", the node will be back online once it reboots. "forever" will
-keep the node in standby after reboot. The life time defaults to
-"forever"."""
-        addtion_usage = " [lifetime]"
-
-    parser = ArgParser(description=usage_template.format(action=action),
-            usage="{} [--all | <node>... ]{}".format(action_type, addtion_usage),
-            add_help=False,
-            formatter_class=RawDescriptionHelpFormatter)
-    parser.add_argument("-h", "--help", action="store_true", dest="help", help="Show this help message")
-    parser.add_argument("--all", help="To {} on all nodes".format(action), action="store_true", dest="all")
-
-    options, args = parse_options(parser, args)
-    if options is None or args is None:
-        raise utils.TerminateSubCommand
-    if options.all and args:
-        context.fatal_error("Should either use --all or specific node(s)")
-
-    # return local node
-    if not options.all and not args:
-        return [utils.this_node()]
-    member_list = utils.list_cluster_nodes()
-    if not member_list:
-        context.fatal_error("Cannot get the node list from cluster")
-    for node in args:
-        if node not in member_list:
-            context.fatal_error("Node \"{}\" is not a cluster node".format(node))
-
-    node_list = member_list if options.all else args
-    for node in node_list:
-        try:
-            utils.ping_node(node)
-        except ValueError as err:
-            logger.warning(str(err))
-            node_list.remove(node)
-    return node_list
-
-
 def _remove_completer(args):
     try:
         n = utils.list_cluster_nodes()
@@ -148,7 +92,7 @@ class Cluster(command.UI):
     @command.skill_level('administrator')
     def do_start(self, context, *args):
         '''
-        Starts the cluster services on all nodes or specific node(s)
+        Starts the cluster stack on all nodes or specific node(s)
         '''
         service_check_list = ["pacemaker.service"]
         start_qdevice = False
@@ -159,37 +103,45 @@ class Cluster(command.UI):
         node_list = parse_option_for_nodes(context, *args)
         for node in node_list[:]:
             if all([utils.service_is_active(srv, remote_addr=node) for srv in service_check_list]):
-                logger.info("Cluster services already started on {}".format(node))
+                logger.info("The cluster stack already started on {}".format(node))
                 node_list.remove(node)
         if not node_list:
             return
 
         if start_qdevice:
             utils.start_service("corosync-qdevice", node_list=node_list)
-        bootstrap.start_pacemaker(node_list)
+        node_list = bootstrap.start_pacemaker(node_list)
         if start_qdevice:
             qdevice.QDevice.check_qdevice_vote()
         for node in node_list:
-            logger.info("Cluster services started on {}".format(node))
+            logger.info("The cluster stack started on {}".format(node))
 
     @command.skill_level('administrator')
     def do_stop(self, context, *args):
         '''
-        Stops the cluster services on all nodes or specific node(s)
+        Stops the cluster stack on all nodes or specific node(s)
         '''
         node_list = parse_option_for_nodes(context, *args)
         for node in node_list[:]:
             if not utils.service_is_active("corosync.service", remote_addr=node):
                 if utils.service_is_active("sbd.service", remote_addr=node):
                     utils.stop_service("corosync", remote_addr=node)
-                    logger.info("Cluster services stopped on {}".format(node))
+                    logger.info("The cluster stack stopped on {}".format(node))
                 else:
-                    logger.info("Cluster services already stopped on {}".format(node))
+                    logger.info("The cluster stack already stopped on {}".format(node))
+                node_list.remove(node)
+            elif not utils.service_is_active("pacemaker.service", remote_addr=node):
+                utils.stop_service("corosync", remote_addr=node)
+                logger.info("The cluster stack stopped on {}".format(node))
                 node_list.remove(node)
         if not node_list:
             return
 
-        if not utils.get_dc(timeout=5):
+        dc_deadtime = utils.get_property("dc-deadtime") or str(constants.DC_DEADTIME_DEFAULT)
+        dc_timeout = int(dc_deadtime.strip('s')) + 5
+        try:
+            utils.check_function_with_timeout(utils.get_dc, wait_timeout=dc_timeout)
+        except TimeoutError:
             logger.error("No DC found currently, please wait if the cluster is still starting")
             return False
 
@@ -200,50 +152,47 @@ class Cluster(command.UI):
             utils.set_dlm_option(enable_quorum_fencing=0, enable_quorum_lockspace=0)
 
         # Stop pacemaker since it can make sure cluster has quorum until stop corosync
-        utils.stop_service("pacemaker", node_list=node_list)
+        node_list = utils.stop_service("pacemaker", node_list=node_list)
         # Then, stop qdevice if is active
         if utils.service_is_active("corosync-qdevice.service"):
             utils.stop_service("corosync-qdevice.service", node_list=node_list)
         # Last, stop corosync
-        utils.stop_service("corosync", node_list=node_list)
+        node_list = utils.stop_service("corosync", node_list=node_list)
 
         for node in node_list:
-            logger.info("Cluster services stopped on {}".format(node))
+            logger.info("The cluster stack stopped on {}".format(node))
 
     @command.skill_level('administrator')
     def do_restart(self, context, *args):
         '''
-        Restarts the cluster services on all nodes or specific node(s)
+        Restarts the cluster stack on all nodes or specific node(s)
         '''
         parse_option_for_nodes(context, *args)
         self.do_stop(context, *args)
         self.do_start(context, *args)
 
-    def _enable_disable_common(self, context, *args):
-        '''
-        Common part for enable and disable
-        '''
-        node_list = parse_option_for_nodes(context, *args)
-        action = context.get_command_name()
-        utils.cluster_run_cmd("systemctl {} pacemaker.service".format(action), node_list)
-        if utils.service_is_available("corosync-qdevice.service") and (utils.is_qdevice_configured() or action == "disable"):
-            utils.cluster_run_cmd("systemctl {} corosync-qdevice.service".format(action), node_list)
-        for node in node_list:
-            logger.info("Cluster services %s on %s", action+'d', node)
-
     @command.skill_level('administrator')
     def do_enable(self, context, *args):
         '''
         Enable the cluster services on this node
         '''
-        self._enable_disable_common(context, *args)
+        node_list = parse_option_for_nodes(context, *args)
+        node_list = utils.enable_service("pacemaker.service", node_list=node_list)
+        if utils.service_is_available("corosync-qdevice.service") and utils.is_qdevice_configured():
+            utils.enable_service("corosync-qdevice.service", node_list=node_list)
+        for node in node_list:
+            logger.info("Cluster services enabled on %s", node)
 
     @command.skill_level('administrator')
     def do_disable(self, context, *args):
         '''
         Disable the cluster services on this node
         '''
-        self._enable_disable_common(context, *args)
+        node_list = parse_option_for_nodes(context, *args)
+        node_list = utils.disable_service("pacemaker.service", node_list=node_list)
+        utils.disable_service("corosync-qdevice.service", node_list=node_list)
+        for node in node_list:
+            logger.info("Cluster services disabled on %s", node)
 
     def _args_implicit(self, context, args, name):
         '''
@@ -271,7 +220,7 @@ class Cluster(command.UI):
         if len(args) > 0:
             if '--dry-run' in args or looks_like_hostnames(args):
                 args = ['--yes', '--nodes'] + [arg for arg in args if arg != '--dry-run']
-        parser = ArgParser(description="""
+        parser = ArgumentParser(description="""
 Initialize a cluster from scratch. This command configures
 a complete cluster, and can also add additional cluster
 nodes to the initial one-node cluster using the --nodes
@@ -316,7 +265,7 @@ Examples:
   # Setup the cluster on the current node, with SBD+OCFS2
   crm cluster init -s <share disk1> -o <share disk2> -y
 
-  # Setup the cluster on the current node, with SBD+OCFS2+Cluster LVM
+  # Setup the cluster on the current node, with SBD++OCFS2++Cluster LVM
   crm cluster init -s <share disk1> -o <share disk2> -o <share disk3> -C -y
 
   # Add SBD on a running cluster
@@ -342,6 +291,8 @@ Examples:
                             help="Enable SBD even if no SBD device is configured (diskless mode)")
         parser.add_argument("-w", "--watchdog", dest="watchdog", metavar="WATCHDOG",
                             help="Use the given watchdog device or driver name")
+        parser.add_argument("-x", "--skip-csync2-sync", dest="skip_csync2", action="store_true",
+                            help="Skip csync2 initialization (an experimental option)")
         parser.add_argument("--no-overwrite-sshkey", action="store_true", dest="no_overwrite_sshkey",
                             help='Avoid "/root/.ssh/id_rsa" overwrite if "-y" option is used (False by default; Deprecated)')
 
@@ -399,6 +350,8 @@ Examples:
             parser.error("Invalid stage (%s)" % (stage))
 
         if options.qnetd_addr:
+            if not utils.service_is_available("corosync-qdevice.service"):
+                utils.fatal("corosync-qdevice.service is not available")
             if options.qdevice_heuristics_mode and not options.qdevice_heuristics:
                 parser.error("Option --qdevice-heuristics is required if want to configure heuristics mode")
             options.qdevice_heuristics_mode = options.qdevice_heuristics_mode or "sync"
@@ -426,7 +379,7 @@ Examples:
         '''
         Join this node to an existing cluster
         '''
-        parser = ArgParser(description="""
+        parser = ArgumentParser(description="""
 Join the current node to an existing cluster. The
 current node cannot be a member of a cluster already.
 Pass any node in the existing cluster as the argument
@@ -471,6 +424,7 @@ Examples:
         join_context.ui_context = context
         join_context.stage = stage
         join_context.type = "join"
+        join_context.validate_option()
 
         bootstrap.bootstrap_join(join_context)
 
@@ -483,7 +437,7 @@ Examples:
         '''
         Remove the given node(s) from the cluster.
         '''
-        parser = ArgParser(description="""
+        parser = ArgumentParser(description="""
 Remove one or more nodes from the cluster.
 
 This command can remove the last node in the cluster,
@@ -569,7 +523,7 @@ the config.core.force option.""",
         * arbitrator IP / hostname (optional)
         * list of tickets (can be empty)
         '''
-        parser = ArgParser(description="""
+        parser = ArgumentParser(description="""
 Create a new geo cluster with the current cluster as the
 first member. Pass the complete geo cluster topology as
 arguments to this command, and then use geo-join and
@@ -632,7 +586,7 @@ Cluster Description
         '''
         Join this cluster to a geo configuration.
         '''
-        parser = ArgParser(description="""
+        parser = ArgumentParser(description="""
 This command should be run from one of the nodes in a cluster
 which is currently not a member of a geo cluster. The geo
 cluster configuration will be fetched from the provided node,
@@ -676,7 +630,7 @@ an existing cluster.""",
         '''
         Make this node a geo arbitrator.
         '''
-        parser = ArgParser(description="""
+        parser = ArgumentParser(description="""
 Configure the current node as a geo arbitrator. The command
 requires an existing geo cluster or geo arbitrator from which
 to get the geo cluster configuration.""",
@@ -735,7 +689,7 @@ to get the geo cluster configuration.""",
         import time
         t0 = time.time()
         timeout = float(timeout)
-        cmd = 'crm_mon -bD1 2&>1 >/dev/null'
+        cmd = 'crm_mon -bD1 >/dev/null 2>&1'
         ret = utils.ext_cmd(cmd)
         while ret in (107, 64) and time.time() < t0 + timeout:
             time.sleep(1)
@@ -764,24 +718,18 @@ to get the geo cluster configuration.""",
             if hosts is None:
                 context.fatal_error("failed to get node list from cluster")
 
-        opts = parallax.Options()
-        opts.ssh_options = ['StrictHostKeyChecking=no']
-        for host in hosts:
-            res = utils.check_ssh_passwd_need(host)
-            if res:
-                opts.askpass = True
-                break
-        for host, result in parallax.call(hosts, cmd, opts).items():
+        for host, result in crmsh.parallax.parallax_run(hosts, cmd).items():
             if isinstance(result, parallax.Error):
-                logger.error("[%s]: %s" % (host, result))
+                logger.error("[%s]: %s", host, result)
             else:
-                if result[0] != 0:
-                    logger.error("[%s]: rc=%s\n%s\n%s" % (host, result[0], utils.to_ascii(result[1]), utils.to_ascii(result[2])))
+                rc, stdout, stderr = result
+                if rc != 0:
+                    logger.error("[%s]: rc=%s\n%s\n%s", host, rc, utils.to_ascii(stdout), utils.to_ascii(stderr))
                 else:
-                    if not result[1]:
+                    if not stdout:
                         logger.info("[%s]" % host)
                     else:
-                        logger.info("[%s]\n%s" % (host, utils.to_ascii(result[1])))
+                        logger.info("[%s]\n%s" % (host, utils.to_ascii(stdout)))
 
     def do_copy(self, context, local_file, *nodes):
         '''
