@@ -47,13 +47,6 @@ class Context(object):
         '''
         Execute the given command line.
         '''
-        def trans_to_help(line):
-            line_list = line.split()
-            if line_list[-1] in ["-h", "--help"] and \
-               line_list[-2] == "property":
-                return " ".join(line_list[:-2] + ["help", "property"])
-            else:
-                return line
         promptstr = "crm({}/{}){}# ".format(main.cib_prompt(), utils.this_node(), self.prompt())
         logger_utils.log_only_to_file("{}{}".format(promptstr, line))
 
@@ -61,7 +54,7 @@ class Context(object):
         if not line or line.startswith('#'):
             return True
 
-        line = trans_to_help(line)
+        line = _try_redirect_help_argument_to_subcommand(line)
 
         self._mark = len(self.stack)
         self._in_transit = False
@@ -89,15 +82,9 @@ class Context(object):
                     cmd = True
                     break
             if cmd:
+                utils.check_user_access(self.current_level().name)
                 rv = self.execute_command() is not False
-        except ValueError as msg:
-            if config.core.debug or options.regression_tests:
-                import traceback
-                traceback.print_exc()
-                sys.stdout.flush()
-            logger.error("%s: %s", self.get_qualified_name(), msg)
-            rv = False
-        except IOError as msg:
+        except (ValueError, IOError) as msg:
             if config.core.debug or options.regression_tests:
                 import traceback
                 traceback.print_exc()
@@ -410,4 +397,21 @@ class Context(object):
         logger.info("%s: %s", self.get_qualified_name(), msg)
 
 
+def _try_redirect_help_argument_to_subcommand(line):
+    tokens = shlex.split(line)
+    if "--help-without-redirect" in tokens:
+        # Help text of some subcommands are read from `--help` generated by argparse.
+        # They should not be redirected, or a circular redirect will be formed.
+        # See crmsh.help._load_help.
+        return ' '.join(("--help" if token == "--help-without-redirect" else token for token in tokens))
+    elif tokens[-1] in ["-h", "--help"]:
+        if len(tokens) == 2 and tokens[0] == 'report':
+            # subcommand report has --help implementation, but _load_help doest not load from it.
+            return line
+        else:
+            return 'help ' + ' '.join(tokens[:-1])
+    else:
+        return line
+
+
 # vim:ts=4:sw=4:et:
