@@ -3,23 +3,54 @@ import time
 import os
 import datetime
 import yaml
+
+import behave
 from behave import given, when, then
-from crmsh import corosync, parallax, sbd
+from crmsh import corosync, parallax, sbd, userdir, bootstrap
 from crmsh import utils as crmutils
 from utils import check_cluster_state, check_service_state, online, run_command, me, \
-                  run_command_local_or_remote, file_in_archive
+                  run_command_local_or_remote, file_in_archive, \
+                  assert_eq, is_unclean
 import const
 
+
+def _parse_str(text):
+    return text[1:-1].encode('utf-8').decode('unicode_escape')
+_parse_str.pattern='".*"'
+
+
+behave.use_step_matcher("cfparse")
+behave.register_type(str=_parse_str)
+
+
 @when('Write multi lines to file "{f}"')
 def step_impl(context, f):
-    with open(f, 'w') as fd:
-        fd.write(context.text)
+    data_list = context.text.split('\n')
+    for line in data_list:
+        echo_option = " -n" if line == data_list[-1] else ""
+        cmd = "echo{} {}|sudo tee -a {}".format(echo_option, line, f)
+        run_command(context, cmd)
+
 
 @given('Cluster service is "{state}" on "{addr}"')
 def step_impl(context, state, addr):
     assert check_cluster_state(context, state, addr) is True
 
 
+@given('Nodes [{nodes:str+}] are cleaned up')
+def step_impl(context, nodes):
+    run_command(context, 'crm resource cleanup || true')
+    for node in nodes:
+        # wait for ssh service
+        for _ in range(10):
+            rc, _, _ = crmutils.get_stdout_stderr('ssh {} true'.format(node))
+            if rc == 0:
+                break
+            time.sleep(1)
+        run_command_local_or_remote(context, "crm cluster stop {} || true".format(node), node)
+        assert check_cluster_state(context, 'stopped', node) is True
+
+
 @given('Service "{name}" is "{state}" on "{addr}"')
 def step_impl(context, name, state, addr):
     assert check_service_state(context, name, state, addr) is True
@@ -27,7 +58,7 @@ def step_impl(context, name, state, addr):
 
 @given('Has disk "{disk}" on "{addr}"')
 def step_impl(context, disk, addr):
-    out = run_command_local_or_remote(context, "fdisk -l", addr)
+    _, out, _ = run_command_local_or_remote(context, "fdisk -l", addr)
     assert re.search(r'{} '.format(disk), out) is not None
 
 
@@ -38,19 +69,19 @@ def step_impl(context, nodelist):
 
 @given('Run "{cmd}" OK')
 def step_impl(context, cmd):
-    rc, _, = run_command(context, cmd)
+    rc, _, _ = run_command(context, cmd)
     assert rc == 0
 
 
 @then('Run "{cmd}" OK')
 def step_impl(context, cmd):
-    rc, _, = run_command(context, cmd)
+    rc, _, _ = run_command(context, cmd)
     assert rc == 0
 
 
 @when('Run "{cmd}" OK')
 def step_impl(context, cmd):
-    rc, _, = run_command(context, cmd)
+    rc, _, _ = run_command(context, cmd)
     assert rc == 0
 
 
@@ -63,8 +94,7 @@ def step_impl(context, addr, iface):
 
 @when('Run "{cmd}" on "{addr}"')
 def step_impl(context, cmd, addr):
-    out = run_command_local_or_remote(context, cmd, addr)
-    context.stdout = out
+    _, out, _ = run_command_local_or_remote(context, cmd, addr)
 
 
 @then('Print stdout')
@@ -74,18 +104,17 @@ def step_impl(context):
 
 @then('Print stderr')
 def step_impl(context):
-    context.logger.info("\n{}".format(context.command_error_output))
+    context.logger.info("\n{}".format(context.stderr))
 
 
 @when('Try "{cmd}" on "{addr}"')
 def step_impl(context, cmd, addr):
-    run_command_local_or_remote(context, cmd, addr, err_record=True)
+    run_command_local_or_remote(context, cmd, addr, exit_on_fail=False)
 
 
 @when('Try "{cmd}"')
 def step_impl(context, cmd):
-    rc, out = run_command(context, cmd, err_record=True)
-    context.return_code = rc
+    _, out, _ = run_command(context, cmd, exit_on_fail=False)
 
 
 @when('Wait "{second}" seconds')
@@ -111,6 +140,12 @@ def step_impl(context, msg):
     context.stdout = None
 
 
+@then('Expected "{msg}" in stderr')
+def step_impl(context, msg):
+    assert msg in context.stderr
+    context.stderr = None
+
+
 @then('Expected regrex "{reg_str}" in stdout')
 def step_impl(context, reg_str):
     res = re.search(reg_str, context.stdout)
@@ -131,14 +166,14 @@ def step_impl(context, msg):
 
 @then('Except "{msg}"')
 def step_impl(context, msg):
-    assert msg in context.command_error_output
-    context.command_error_output = None
+    assert msg in context.stderr
+    context.stderr = None
 
 
 @then('Except multiple lines')
 def step_impl(context):
-    assert context.command_error_output.split('\n') == context.text.split('\n')
-    context.command_error_output = None
+    assert context.text in context.stderr
+    context.stderr = None
 
 
 @then('Expected multiple lines in output')
@@ -149,8 +184,8 @@ def step_impl(context):
 
 @then('Except "{msg}" in stderr')
 def step_impl(context, msg):
-    assert msg in context.command_error_output
-    context.command_error_output = None
+    assert msg in context.stderr
+    context.stderr = None
 
 
 @then('Cluster service is "{state}" on "{addr}"')
@@ -180,20 +215,20 @@ def step_impl(context, node):
 
 @then('IP "{addr}" is used by corosync on "{node}"')
 def step_impl(context, addr, node):
-    out = run_command_local_or_remote(context, 'corosync-cfgtool -s', node)
+    _, out, _ = run_command_local_or_remote(context, 'corosync-cfgtool -s', node)
     res = re.search(r' {}\n'.format(addr), out)
     assert bool(res) is True
 
 
 @then('Cluster name is "{name}"')
 def step_impl(context, name):
-    _, out = run_command(context, 'corosync-cmapctl -b totem.cluster_name')
+    _, out, _ = run_command(context, 'corosync-cmapctl -b totem.cluster_name')
     assert out.split()[-1] == name
 
 
 @then('Cluster virtual IP is "{addr}"')
 def step_impl(context, addr):
-    _, out = run_command(context, 'crm configure show|grep -A1 IPaddr2')
+    _, out, _ = run_command(context, 'crm configure show|grep -A1 IPaddr2')
     res = re.search(r' ip={}'.format(addr), out)
     assert bool(res) is True
 
@@ -205,35 +240,35 @@ def step_impl(context):
 
 @then('Show cluster status on "{addr}"')
 def step_impl(context, addr):
-    out = run_command_local_or_remote(context, 'crm_mon -1', addr)
+    _, out, _ = run_command_local_or_remote(context, 'crm_mon -1', addr)
     if out:
         context.logger.info("\n{}".format(out))
 
 
 @then('Show corosync ring status')
 def step_impl(context):
-    _, out = run_command(context, 'crm corosync status ring')
+    _, out, _ = run_command(context, 'crm corosync status ring')
     if out:
         context.logger.info("\n{}".format(out))
 
 
 @then('Show crm configure')
 def step_impl(context):
-    _, out = run_command(context, 'crm configure show')
+    _, out, _ = run_command(context, 'crm configure show')
     if out:
         context.logger.info("\n{}".format(out))
 
 
 @then('Show status from qnetd')
 def step_impl(context):
-    _, out = run_command(context, 'crm corosync status qnetd')
+    _, out, _ = run_command(context, 'crm corosync status qnetd')
     if out:
         context.logger.info("\n{}".format(out))
 
 
 @then('Show corosync qdevice configuration')
 def step_impl(context):
-    _, out = run_command(context, "sed -n -e '/quorum/,/^}/ p' /etc/corosync/corosync.conf")
+    _, out, _ = run_command(context, "sed -n -e '/quorum/,/^}/ p' /etc/corosync/corosync.conf")
     if out:
         context.logger.info("\n{}".format(out))
 
@@ -242,9 +277,9 @@ def step_impl(context):
 def step_impl(context, res, res_type, state):
     try_count = 0
     result = None
-    while try_count < 5:
+    while try_count < 20:
         time.sleep(1)
-        _, out = run_command(context, "crm_mon -1rR")
+        _, out, _ = run_command(context, "crm_mon -1rR")
         if out:
             result = re.search(r'\s{}\s+.*:+{}\):\s+{} '.format(res, res_type, state), out)
             if not result:
@@ -257,7 +292,7 @@ def step_impl(context, res, res_type, state):
 @then('Resource "{res}" failcount on "{node}" is "{number}"')
 def step_impl(context, res, node, number):
     cmd = "crm resource failcount {} show {}".format(res, node)
-    _, out = run_command(context, cmd)
+    _, out, _ = run_command(context, cmd)
     if out:
         result = re.search(r'name=fail-count-{} value={}'.format(res, number), out)
         assert result is not None
@@ -265,7 +300,7 @@ def step_impl(context, res, node, number):
 
 @then('Resource "{res_type}" not configured')
 def step_impl(context, res_type):
-    _, out = run_command(context, "crm configure show")
+    _, out, _ = run_command(context, "crm configure show")
     result = re.search(r' {} '.format(res_type), out)
     assert result is None
 
@@ -281,7 +316,7 @@ def step_impl(context, cmd):
     cmd_help["crm_cluster_geo-join"] = const.CRM_CLUSTER_GEO_JOIN_H_OUTPUT
     cmd_help["crm_cluster_geo-init-arbitrator"] = const.CRM_CLUSTER_GEO_INIT_ARBIT_H_OUTPUT
     key = '_'.join(cmd.split())
-    assert context.stdout == cmd_help[key]
+    assert_eq(cmd_help[key], context.stdout)
 
 
 @then('Corosync working on "{transport_type}" mode')
@@ -289,12 +324,12 @@ def step_impl(context, transport_type):
     if transport_type == "multicast":
         assert corosync.get_value("totem.transport") is None
     if transport_type == "unicast":
-        assert corosync.get_value("totem.transport") == "udpu"
+        assert_eq("udpu", corosync.get_value("totem.transport"))
 
 
 @then('Expected votes will be "{votes}"')
 def step_impl(context, votes):
-    assert int(corosync.get_value("quorum.expected_votes")) == int(votes)
+    assert_eq(int(votes), int(corosync.get_value("quorum.expected_votes")))
 
 
 @then('Default crm_report tar file created')
@@ -314,6 +349,11 @@ def step_impl(context, f, archive):
     assert file_in_archive(f, archive) is True
 
 
+@then('Directory "{f}" in "{archive}"')
+def step_impl(context, f, archive):
+    assert file_in_archive(f, archive) is True
+
+
 @then('File "{f}" not in "{archive}"')
 def step_impl(context, f, archive):
     assert file_in_archive(f, archive) is False
@@ -322,8 +362,8 @@ def step_impl(context, f, archive):
 @then('File "{f}" was synced in cluster')
 def step_impl(context, f):
     cmd = "crm cluster diff {}".format(f)
-    rc, out = run_command(context, cmd)
-    assert out == ""
+    rc, out, _ = run_command(context, cmd)
+    assert_eq("", out)
 
 
 @given('Resource "{res_id}" is started on "{node}"')
@@ -341,31 +381,39 @@ def step_impl(context, res_id, node):
 @then('SBD option "{key}" value is "{value}"')
 def step_impl(context, key, value):
     res = sbd.SBDManager.get_sbd_value_from_config(key)
-    assert res == value
+    assert_eq(value, res)
 
 
 @then('SBD option "{key}" value for "{dev}" is "{value}"')
 def step_impl(context, key, dev, value):
     res = sbd.SBDTimeout.get_sbd_msgwait(dev)
-    assert res == int(value)
+    assert_eq(int(value), res)
 
 
 @then('Cluster property "{key}" is "{value}"')
 def step_impl(context, key, value):
     res = crmutils.get_property(key)
-    assert res is not None and str(res) == value
+    assert res is not None
+    assert_eq(value, str(res))
+
+
+@then('Property "{key}" in "{type}" is "{value}"')
+def step_impl(context, key, type, value):
+    res = crmutils.get_property(key, type)
+    assert res is not None
+    assert_eq(value, str(res))
 
 
 @then('Parameter "{param_name}" not configured in "{res_id}"')
 def step_impl(context, param_name, res_id):
-    _, out = run_command(context, "crm configure show {}".format(res_id))
+    _, out, _ = run_command(context, "crm configure show {}".format(res_id))
     result = re.search("params {}=".format(param_name), out)
     assert result is None
 
 
 @then('Parameter "{param_name}" configured in "{res_id}"')
 def step_impl(context, param_name, res_id):
-    _, out = run_command(context, "crm configure show {}".format(res_id))
+    _, out, _ = run_command(context, "crm configure show {}".format(res_id))
     result = re.search("params {}=".format(param_name), out)
     assert result is not None
 
@@ -376,7 +424,7 @@ def step_impl(context, path, value):
     with open(yaml_file) as f:
         data = yaml.load(f, Loader=yaml.SafeLoader)
     sec_name, key = path.split(':')
-    assert str(data[sec_name][key]) == str(value)
+    assert_eq(str(value), str(data[sec_name][key]))
 
 
 @when('Wait for DC')
@@ -385,3 +433,68 @@ def step_impl(context):
         time.sleep(1)
         if crmutils.get_dc():
             break
+
+
+@when('Run parallax call many times on "{node}"')
+def step_impl(context, node):
+    import resource
+    resource.setrlimit(resource.RLIMIT_NOFILE, (50, 50))
+    for i in range(51):
+        parallax.parallax_call([node], "true")
+
+
+@then('File "{path}" exists on "{node}"')
+def step_impl(context, path, node):
+    parallax.parallax_call([node], 'sudo test -f {}'.format(path))
+
+
+@then('File "{path}" not exist on "{node}"')
+def step_impl(context, path, node):
+    parallax.parallax_call([node], '[ ! -f {} ]'.format(path))
+
+
+@then('Directory "{path}" is empty on "{node}"')
+def step_impl(context, path, node):
+    parallax.parallax_call([node], '[ ! "$(ls -A {})" ]'.format(path))
+
+
+@then('Directory "{path}" not empty on "{node}"')
+def step_impl(context, path, node):
+    parallax.parallax_call([node], '[ "$(ls -A {})" ]'.format(path))
+
+
+@then('Node "{node}" is UNCLEAN')
+def step_impl(context, node):
+    assert is_unclean(node) is True
+
+
+@then('Wait "{count}" seconds for "{node}" successfully fenced')
+def step_impl(context, count, node):
+    index = 0
+    while index <= int(count):
+        rc, out, _ = crmutils.get_stdout_stderr("stonith_admin -h {}".format(node))
+        if "Node {} last fenced at:".format(node) in out:
+            return True
+        time.sleep(1)
+        index += 1
+    return False
+
+@then('Check passwordless for hacluster between "{nodelist}"')
+def step_impl(context, nodelist):
+    if userdir.getuser() != 'root':
+        return True
+    need_pw_list = crmutils.check_passwordless_between_nodes(nodelist.split(), 'hacluster')
+    for m, n in need_pw_list:
+        context.logger.error(f"There is no passwordless configured from {m} to {n} under 'hacluster'")
+    assert need_pw_list == []
+
+
+@then('Check user shell for hacluster between "{nodelist}"')
+def step_impl(context, nodelist):
+    if userdir.getuser() != 'root' or userdir.get_sudoer():
+        return True
+    for node in nodelist.split():
+        if node == me():
+            assert bootstrap.is_nologin('hacluster') is False
+        else:
+            assert bootstrap.is_nologin('hacluster', node) is False
