@@ -29,7 +29,6 @@ class TestSBDTimeout(unittest.TestCase):
         _dict = {"sbd.watchdog_timeout": 5, "sbd.msgwait": 10}
         _inst_q = mock.Mock()
         self.sbd_timeout_inst = sbd.SBDTimeout(mock.Mock(profiles_dict=_dict, is_s390=True, qdevice_inst=_inst_q))
-        self.sbd_timeout_inst_removing = sbd.SBDTimeout(mock.Mock(), True)
 
     def tearDown(self):
         """
@@ -42,15 +41,24 @@ class TestSBDTimeout(unittest.TestCase):
         Global tearDown.
         """
 
+    def test_initialize_timeout(self):
+        self.sbd_timeout_inst._set_sbd_watchdog_timeout = mock.Mock()
+        self.sbd_timeout_inst._set_sbd_msgwait = mock.Mock()
+        self.sbd_timeout_inst._adjust_sbd_watchdog_timeout_with_diskless_and_qdevice = mock.Mock()
+        self.sbd_timeout_inst.initialize_timeout()
+        self.sbd_timeout_inst._set_sbd_watchdog_timeout.assert_called_once()
+        self.sbd_timeout_inst._set_sbd_msgwait.assert_not_called()
+        self.sbd_timeout_inst._adjust_sbd_watchdog_timeout_with_diskless_and_qdevice.assert_called_once()
+
     @mock.patch('logging.Logger.warning')
     def test_set_sbd_watchdog_timeout(self, mock_warn):
-        self.sbd_timeout_inst.set_sbd_watchdog_timeout()
+        self.sbd_timeout_inst._set_sbd_watchdog_timeout()
         mock_warn.assert_called_once_with("sbd_watchdog_timeout is set to %d for s390, it was %d", sbd.SBDTimeout.SBD_WATCHDOG_TIMEOUT_DEFAULT_S390, 5)
 
     @mock.patch('logging.Logger.warning')
     def test_set_sbd_msgwait(self, mock_warn):
         self.sbd_timeout_inst.sbd_watchdog_timeout = 15
-        self.sbd_timeout_inst.set_sbd_msgwait()
+        self.sbd_timeout_inst._set_sbd_msgwait()
         mock_warn.assert_called_once_with("sbd msgwait is set to %d, it was %d", 30, 10)
 
     @mock.patch('logging.Logger.warning')
@@ -62,7 +70,7 @@ class TestSBDTimeout(unittest.TestCase):
         mock_is_active.return_value = True
         mock_get_sync.return_value = 15
         self.sbd_timeout_inst.sbd_watchdog_timeout = 5
-        self.sbd_timeout_inst.adjust_sbd_watchdog_timeout_with_diskless_and_qdevice()
+        self.sbd_timeout_inst._adjust_sbd_watchdog_timeout_with_diskless_and_qdevice()
         mock_warn.assert_called_once_with("sbd_watchdog_timeout is set to 20 for qdevice, it was 5")
 
     @mock.patch('logging.Logger.warning')
@@ -70,7 +78,7 @@ class TestSBDTimeout(unittest.TestCase):
     def test_adjust_sbd_watchdog_timeout_with_diskless_and_qdevice_all(self, mock_is_configured, mock_warn):
         mock_is_configured.return_value = False
         self.sbd_timeout_inst.sbd_watchdog_timeout = 5
-        self.sbd_timeout_inst.adjust_sbd_watchdog_timeout_with_diskless_and_qdevice()
+        self.sbd_timeout_inst._adjust_sbd_watchdog_timeout_with_diskless_and_qdevice()
         mock_warn.assert_called_once_with("sbd_watchdog_timeout is set to 35 for qdevice, it was 5")
 
     @mock.patch('crmsh.utils.get_stdout_or_raise_error')
@@ -141,7 +149,7 @@ class TestSBDTimeout(unittest.TestCase):
 
         self.sbd_timeout_inst._load_configurations()
 
-        mock_2node.assert_called_once_with(False)
+        mock_2node.assert_called_once_with()
         mock_get_sbd_dev.assert_called_once_with()
         mock_get_msgwait.assert_called_once_with("/dev/sda1")
         mock_pcmk_delay.assert_called_once_with(True)
@@ -164,7 +172,7 @@ class TestSBDTimeout(unittest.TestCase):
 
         self.sbd_timeout_inst._load_configurations()
 
-        mock_2node.assert_called_once_with(False)
+        mock_2node.assert_called_once_with()
         mock_get_sbd_dev.assert_called_once_with()
         mock_get_watchdog_timeout.assert_called_once_with()
         mock_get_stonith_watchdog_timeout.assert_called_once_with()
@@ -247,7 +255,7 @@ class TestSBDTimeout(unittest.TestCase):
         mock_mkdirp.assert_not_called()
 
     @mock.patch('crmsh.utils.cluster_run_cmd')
-    @mock.patch('crmsh.bootstrap.csync2_update')
+    @mock.patch('crmsh.bootstrap.sync_file')
     @mock.patch('crmsh.utils.str2file')
     @mock.patch('crmsh.utils.mkdirp')
     @mock.patch('crmsh.utils.get_systemd_timeout_start_in_sec')
@@ -327,13 +335,15 @@ class TestSBDManager(unittest.TestCase):
         mock_status.assert_called_once_with(sbd.SBDManager.SBD_STATUS_DESCRIPTION)
         mock_warn.assert_called_once_with("Not configuring SBD - STONITH will be disabled.")
 
+    @mock.patch('crmsh.sbd.SBDManager._no_overwrite_check')
     @mock.patch('crmsh.sbd.SBDManager._get_sbd_device_from_config')
     @mock.patch('crmsh.bootstrap.confirm')
     @mock.patch('logging.Logger.info')
-    def test_get_sbd_device_interactive_already_configured(self, mock_status, mock_confirm, mock_from_config):
+    def test_get_sbd_device_interactive_already_configured(self, mock_status, mock_confirm, mock_from_config, mock_no_overwrite):
         self.sbd_inst._context = mock.Mock(yes_to_all=False)
-        mock_confirm.side_effect = [True, False]
+        mock_confirm.return_value = True
         mock_from_config.return_value = ["/dev/sda1"]
+        mock_no_overwrite.return_value = True
 
         res = self.sbd_inst._get_sbd_device_interactive()
         self.assertEqual(res, ["/dev/sda1"])
@@ -341,19 +351,20 @@ class TestSBDManager(unittest.TestCase):
         mock_status.assert_called_once_with(sbd.SBDManager.SBD_STATUS_DESCRIPTION)
         mock_confirm.assert_has_calls([
             mock.call("Do you wish to use SBD?"),
-            mock.call("SBD is already configured to use /dev/sda1 - overwrite?")
             ])
         mock_status.assert_called_once_with(sbd.SBDManager.SBD_STATUS_DESCRIPTION)
         mock_from_config.assert_called_once_with()
 
     @mock.patch('crmsh.bootstrap.prompt_for_string')
+    @mock.patch('crmsh.sbd.SBDManager._no_overwrite_check')
     @mock.patch('crmsh.sbd.SBDManager._get_sbd_device_from_config')
     @mock.patch('crmsh.bootstrap.confirm')
     @mock.patch('logging.Logger.info')
-    def test_get_sbd_device_interactive_diskless(self, mock_status, mock_confirm, mock_from_config, mock_prompt):
+    def test_get_sbd_device_interactive_diskless(self, mock_status, mock_confirm, mock_from_config, mock_no_overwrite, mock_prompt):
         self.sbd_inst._context = mock.Mock(yes_to_all=False)
         mock_confirm.return_value = True
-        mock_from_config.return_value = None
+        mock_no_overwrite.return_value = False
+        mock_from_config.return_value = []
         mock_prompt.return_value = "none"
 
         self.sbd_inst._get_sbd_device_interactive()
@@ -363,14 +374,16 @@ class TestSBDManager(unittest.TestCase):
         mock_prompt.assert_called_once_with('Path to storage device (e.g. /dev/disk/by-id/...), or "none" for diskless sbd, use ";" as separator for multi path', 'none|\\/.*')
 
     @mock.patch('crmsh.bootstrap.prompt_for_string')
+    @mock.patch('crmsh.sbd.SBDManager._no_overwrite_check')
     @mock.patch('crmsh.sbd.SBDManager._get_sbd_device_from_config')
     @mock.patch('crmsh.bootstrap.confirm')
     @mock.patch('logging.Logger.info')
-    def test_get_sbd_device_interactive_null_and_diskless(self, mock_status, mock_confirm, mock_from_config, mock_prompt):
+    def test_get_sbd_device_interactive_null_and_diskless(self, mock_status, mock_confirm, mock_from_config, mock_no_overwrite, mock_prompt):
         self.sbd_inst._context = mock.Mock(yes_to_all=False)
         mock_confirm.return_value = True
-        mock_from_config.return_value = None
-        mock_prompt.side_effect = [None, "none"]
+        mock_no_overwrite.return_value = False
+        mock_from_config.return_value = []
+        mock_prompt.return_value = "none"
 
         self.sbd_inst._get_sbd_device_interactive()
 
@@ -378,7 +391,7 @@ class TestSBDManager(unittest.TestCase):
         mock_confirm.assert_called_once_with("Do you wish to use SBD?")
         mock_from_config.assert_called_once_with()
         mock_prompt.assert_has_calls([
-            mock.call('Path to storage device (e.g. /dev/disk/by-id/...), or "none" for diskless sbd, use ";" as separator for multi path', 'none|\\/.*') for x in range(2)
+            mock.call('Path to storage device (e.g. /dev/disk/by-id/...), or "none" for diskless sbd, use ";" as separator for multi path', 'none|\\/.*')
             ])
 
     @mock.patch('crmsh.utils.re_split_string')
@@ -386,13 +399,15 @@ class TestSBDManager(unittest.TestCase):
     @mock.patch('logging.Logger.error')
     @mock.patch('crmsh.sbd.SBDManager._verify_sbd_device')
     @mock.patch('crmsh.bootstrap.prompt_for_string')
+    @mock.patch('crmsh.sbd.SBDManager._no_overwrite_check')
     @mock.patch('crmsh.sbd.SBDManager._get_sbd_device_from_config')
     @mock.patch('crmsh.bootstrap.confirm')
     @mock.patch('logging.Logger.info')
-    def test_get_sbd_device_interactive(self, mock_status, mock_confirm, mock_from_config, mock_prompt, mock_verify, mock_error_msg, mock_warn, mock_split):
+    def test_get_sbd_device_interactive(self, mock_status, mock_confirm, mock_from_config, mock_no_overwrite, mock_prompt, mock_verify, mock_error_msg, mock_warn, mock_split):
         self.sbd_inst._context = mock.Mock(yes_to_all=False)
         mock_confirm.side_effect = [True, False, True]
-        mock_from_config.return_value = None
+        mock_from_config.return_value = []
+        mock_no_overwrite.return_value = False
         mock_prompt.side_effect = ["/dev/test1", "/dev/sda1", "/dev/sdb1"]
         mock_split.side_effect = [["/dev/test1"], ["/dev/sda1"], ["/dev/sdb1"]]
         mock_verify.side_effect = [ValueError("/dev/test1 error"), None, None]
@@ -463,12 +478,10 @@ class TestSBDManager(unittest.TestCase):
     def test_initialize_sbd_return(self, mock_info, mock_sbd_timeout):
         mock_inst = mock.Mock()
         mock_sbd_timeout.return_value = mock_inst
-        mock_inst.set_sbd_watchdog_timeout = mock.Mock()
-        mock_inst.adjust_sbd_watchdog_timeout_with_diskless_and_qdevice = mock.Mock()
         self.sbd_inst_diskless._context = mock.Mock(profiles_dict={})
         self.sbd_inst_diskless._initialize_sbd()
         mock_info.assert_called_once_with("Configuring diskless SBD")
-        mock_inst.adjust_sbd_watchdog_timeout_with_diskless_and_qdevice.assert_called_once_with()
+        mock_inst.initialize_timeout.assert_called_once_with()
 
     @mock.patch('crmsh.utils.fatal')
     @mock.patch('crmsh.bootstrap.invoke')
@@ -492,7 +505,7 @@ class TestSBDManager(unittest.TestCase):
             ])
         mock_error.assert_called_once_with("Failed to initialize SBD device /dev/sdc1: error")
 
-    @mock.patch('crmsh.bootstrap.csync2_update')
+    @mock.patch('crmsh.bootstrap.sync_file')
     @mock.patch('crmsh.utils.sysconfig_set')
     @mock.patch('shutil.copyfile')
     def test_update_configuration(self, mock_copy, mock_sysconfig, mock_update):
@@ -683,12 +696,12 @@ class TestSBDManager(unittest.TestCase):
         mock_enabled.assert_called_once_with("sbd.service")
         mock_configured.assert_called_once_with(sbd.SBDManager.SBD_RA)
         mock_run.assert_called_once_with("crm configure primitive {} {}".format(sbd.SBDManager.SBD_RA_ID, sbd.SBDManager.SBD_RA))
-        mock_set_property.assert_called_once_with(stonith_enabled="true")
+        mock_set_property.assert_called_once_with("stonith-enabled", "true")
 
     @mock.patch('crmsh.utils.package_is_installed')
     def test_join_sbd_config_not_installed(self, mock_package):
         mock_package.return_value = False
-        self.sbd_inst.join_sbd("node1")
+        self.sbd_inst.join_sbd("alice", "node1")
         mock_package.assert_called_once_with("sbd")
 
     @mock.patch('crmsh.bootstrap.invoke')
@@ -697,7 +710,7 @@ class TestSBDManager(unittest.TestCase):
     def test_join_sbd_config_not_exist(self, mock_package, mock_exists, mock_invoke):
         mock_package.return_value = True
         mock_exists.return_value = False
-        self.sbd_inst.join_sbd("node1")
+        self.sbd_inst.join_sbd("alice", "node1")
         mock_package.assert_called_once_with("sbd")
         mock_exists.assert_called_once_with("/etc/sysconfig/sbd")
         mock_invoke.assert_called_once_with("systemctl disable sbd.service")
@@ -711,14 +724,13 @@ class TestSBDManager(unittest.TestCase):
         mock_exists.return_value = True
         mock_enabled.return_value = False
 
-        self.sbd_inst.join_sbd("node1")
+        self.sbd_inst.join_sbd("alice", "node1")
 
         mock_package.assert_called_once_with("sbd")
         mock_exists.assert_called_once_with("/etc/sysconfig/sbd")
         mock_invoke.assert_called_once_with("systemctl disable sbd.service")
         mock_enabled.assert_called_once_with("sbd.service", "node1")
 
-    @mock.patch('crmsh.utils.sysconfig_set')
     @mock.patch('logging.Logger.info')
     @mock.patch('crmsh.sbd.SBDManager._verify_sbd_device')
     @mock.patch('crmsh.sbd.SBDManager._get_sbd_device_from_config')
@@ -727,7 +739,7 @@ class TestSBDManager(unittest.TestCase):
     @mock.patch('crmsh.utils.service_is_enabled')
     @mock.patch('os.path.exists')
     @mock.patch('crmsh.utils.package_is_installed')
-    def test_join_sbd(self, mock_package, mock_exists, mock_enabled, mock_invoke, mock_watchdog, mock_get_device, mock_verify, mock_status, mock_set):
+    def test_join_sbd(self, mock_package, mock_exists, mock_enabled, mock_invoke, mock_watchdog, mock_get_device, mock_verify, mock_status):
         mock_package.return_value = True
         mock_exists.return_value = True
         mock_enabled.return_value = True
@@ -736,7 +748,7 @@ class TestSBDManager(unittest.TestCase):
         mock_watchdog.return_value = mock_watchdog_inst
         mock_watchdog_inst.join_watchdog = mock.Mock()
 
-        self.sbd_inst.join_sbd("node1")
+        self.sbd_inst.join_sbd("alice", "node1")
 
         mock_package.assert_called_once_with("sbd")
         mock_exists.assert_called_once_with("/etc/sysconfig/sbd")
@@ -745,7 +757,7 @@ class TestSBDManager(unittest.TestCase):
         mock_verify.assert_called_once_with(["/dev/sdb1"], ["node1"])
         mock_enabled.assert_called_once_with("sbd.service", "node1")
         mock_status.assert_called_once_with("Got SBD configuration")
-        mock_watchdog.assert_called_once_with(peer_host="node1")
+        mock_watchdog.assert_called_once_with(remote_user="alice", peer_host="node1")
         mock_watchdog_inst.join_watchdog.assert_called_once_with()
 
     @mock.patch('crmsh.utils.sysconfig_set')
@@ -766,7 +778,7 @@ class TestSBDManager(unittest.TestCase):
         mock_watchdog.return_value = mock_watchdog_inst
         mock_watchdog_inst.join_watchdog = mock.Mock()
 
-        self.sbd_inst.join_sbd("node1")
+        self.sbd_inst.join_sbd("alice", "node1")
 
         mock_package.assert_called_once_with("sbd")
         mock_exists.assert_called_once_with("/etc/sysconfig/sbd")
@@ -775,7 +787,7 @@ class TestSBDManager(unittest.TestCase):
         mock_warn.assert_called_once_with("node1")
         mock_enabled.assert_called_once_with("sbd.service", "node1")
         mock_status.assert_called_once_with("Got diskless SBD configuration")
-        mock_watchdog.assert_called_once_with(peer_host="node1")
+        mock_watchdog.assert_called_once_with(remote_user="alice", peer_host="node1")
         mock_watchdog_inst.join_watchdog.assert_called_once_with()
 
     @mock.patch('crmsh.sbd.SBDManager._get_sbd_device_from_config')
@@ -869,7 +881,7 @@ class TestSBDManager(unittest.TestCase):
         mock_context.assert_called_once_with()
         mock_get_sbd.assert_called_once_with()
 
-    @mock.patch('crmsh.bootstrap.csync2_update')
+    @mock.patch('crmsh.bootstrap.sync_file')
     @mock.patch('crmsh.utils.sysconfig_set')
     def test_update_configuration_static(self, mock_config_set, mock_csync2):
         sbd_config_dict = {
