@@ -8,12 +8,15 @@ import os
 import socket
 import re
 import imp
+import subprocess
 import unittest
 import pytest
 import logging
 from unittest import mock
 from itertools import chain
-from crmsh import utils, config, tmpfiles, constants
+
+import crmsh.utils
+from crmsh import utils, config, tmpfiles, constants, parallax
 
 logging.basicConfig(level=logging.DEBUG)
 
@@ -31,24 +34,6 @@ def test_print_cluster_nodes(mock_run):
     mock_run.assert_called_once_with("crm_node -l")
 
 
-@mock.patch("crmsh.utils.to_ascii")
-@mock.patch("crmsh.parallax.parallax_call")
-@mock.patch("crmsh.utils.check_ssh_passwd_need")
-def test_run_cmd_on_remote(mock_check_ssh_pw, mock_parallax_call, mock_to_ascii):
-    mock_check_ssh_pw.return_value = False
-    mock_parallax_call.side_effect = ValueError("Exited with error code 2, Error output: cmd error message")
-    mock_to_ascii.return_value = None
-
-    rc, out, err = utils.run_cmd_on_remote("cmd", "other_node")
-    assert rc == 2
-    assert out is None
-    assert err == "cmd error message"
-
-    mock_check_ssh_pw.assert_called_once_with("other_node")
-    mock_parallax_call.assert_called_once_with(["other_node"], "cmd", False)
-    mock_to_ascii.assert_called_once_with(None)
-
-
 @mock.patch("crmsh.utils.get_stdout")
 def test_package_is_installed_local(mock_run):
     mock_run.return_value = (0, None)
@@ -57,17 +42,6 @@ def test_package_is_installed_local(mock_run):
     mock_run.assert_called_once_with("rpm -q --quiet crmsh")
 
 
-@mock.patch("crmsh.utils.run_cmd_on_remote")
-def test_package_is_installed_remote(mock_run_remote):
-    mock_run_remote.return_value = (0, None, None)
-    res = utils.package_is_installed("crmsh", "other_node")
-    assert res is True
-    mock_run_remote.assert_called_once_with(
-            "rpm -q --quiet crmsh",
-            "other_node",
-            "Check whether crmsh is installed on other_node")
-
-
 @mock.patch('crmsh.utils.detect_file')
 def test_check_file_content_included_target_not_exist(mock_detect):
     mock_detect.side_effect = [True, False]
@@ -160,12 +134,15 @@ def test_get_nodeid_from_name(mock_get_stdout, mock_re_search):
     mock_re_search_inst.group.assert_called_once_with(1)
 
 
-@mock.patch('crmsh.utils.get_stdout_stderr')
-def test_check_ssh_passwd_need(mock_run):
-    mock_run.return_value = (1, None, None)
-    res = utils.check_ssh_passwd_need("node1")
+@mock.patch('crmsh.utils.su_get_stdout_stderr')
+def test_check_ssh_passwd_need(mock_su_get_stdout_stderr):
+    mock_su_get_stdout_stderr.return_value = (1, None, None)
+    res = utils.check_ssh_passwd_need("bob", "alice", "node1")
     assert res is True
-    mock_run.assert_called_once_with("ssh -o StrictHostKeyChecking=no -o EscapeChar=none -o ConnectTimeout=15 -T -o Batchmode=yes node1 true")
+    mock_su_get_stdout_stderr.assert_called_once_with(
+        "bob",
+        "ssh -o StrictHostKeyChecking=no -o EscapeChar=none -o ConnectTimeout=15 -T -o Batchmode=yes alice@node1 true",
+    )
 
 
 @mock.patch('logging.Logger.debug')
@@ -203,15 +180,6 @@ def test_cluster_run_cmd_exception(mock_list_nodes):
     mock_list_nodes.assert_called_once_with()
 
 
-@mock.patch('crmsh.parallax.parallax_call')
-@mock.patch('crmsh.utils.list_cluster_nodes')
-def test_cluster_run_cmd(mock_list_nodes, mock_parallax_call):
-    mock_list_nodes.return_value = ["node1", "node2"]
-    utils.cluster_run_cmd("test")
-    mock_list_nodes.assert_called_once_with()
-    mock_parallax_call.assert_called_once_with(["node1", "node2"], "test")
-
-
 @mock.patch('crmsh.utils.list_cluster_nodes')
 def test_list_cluster_nodes_except_me_exception(mock_list_nodes):
     mock_list_nodes.return_value = None
@@ -1029,148 +997,90 @@ class TestServiceManager(unittest.TestCase):
     """
     Unitary tests for class utils.ServiceManager
     """
+    @mock.patch('crmsh.utils.ServiceManager._call_with_parallax')
+    @mock.patch('crmsh.utils.ServiceManager._run_on_single_host')
+    def test_call_single_node(
+            self,
+            mock_run_on_single_host:mock.MagicMock,
+            mock_call_with_parallax: mock.MagicMock,
+    ):
+        mock_run_on_single_host.return_value = 0
+        self.assertEqual(['node1'], utils.ServiceManager._call('node1', list(), 'foo'))
+        mock_run_on_single_host.assert_called_once_with('foo', 'node1')
+        mock_call_with_parallax.assert_not_called()
 
-    @classmethod
-    def setUpClass(cls):
-        """
-        Global setUp.
-        """
+    @mock.patch('crmsh.utils.ServiceManager._call_with_parallax')
+    @mock.patch('crmsh.utils.ServiceManager._run_on_single_host')
+    def test_call_single_node_failure(
+            self,
+            mock_run_on_single_host:mock.MagicMock,
+            mock_call_with_parallax: mock.MagicMock,
+    ):
+        mock_run_on_single_host.return_value = 1
+        self.assertEqual(list(), utils.ServiceManager._call('node1', list(), 'foo'))
+        mock_run_on_single_host.assert_called_once_with('foo', 'node1')
+        mock_call_with_parallax.assert_not_called()
 
-    def setUp(self):
-        """
-        Test setUp.
-        """
-        self.service_local = utils.ServiceManager("service1")
-        self.service_remote = utils.ServiceManager("service1", "node1")
+    @mock.patch('crmsh.utils.ServiceManager._call_with_parallax')
+    @mock.patch('crmsh.utils.ServiceManager._run_on_single_host')
+    def test_call_multiple_node(
+            self,
+            mock_run_on_single_host: mock.MagicMock,
+            mock_call_with_parallax: mock.MagicMock,
+    ):
+        mock_call_with_parallax.return_value = {'node1': (0, '', ''), 'node2': (1, 'out', 'err')}
+        self.assertEqual(['node1'], utils.ServiceManager._call(None, ['node1', 'node2'], 'foo'))
+        mock_run_on_single_host.assert_not_called()
+        mock_call_with_parallax.assert_called_once_with('foo', ['node1', 'node2'])
 
-    def tearDown(self):
-        """
-        Test tearDown.
-        """
+    @mock.patch('crmsh.utils.get_stdout_stderr_auto_ssh_no_input')
+    def test_run_on_single_host_return_1(self, mock_run: mock.MagicMock):
+        mock_run.return_value = (1, 'bar', 'err')
+        self.assertEqual(1, crmsh.utils.ServiceManager._run_on_single_host('foo', 'node1'))
+        mock_run.assert_called_once_with('node1', 'foo')
 
-    @classmethod
-    def tearDownClass(cls):
-        """
-        Global tearDown.
-        """
+    @mock.patch('crmsh.utils.get_stdout_stderr_auto_ssh_no_input')
+    def test_run_on_single_host_return_255(self, mock_run: mock.MagicMock):
+        mock_run.return_value = (255, 'bar', 'err')
+        with self.assertRaises(ValueError):
+            crmsh.utils.ServiceManager._run_on_single_host('foo', 'node1')
+        mock_run.assert_called_once_with('node1', 'foo')
 
-    def test_do_action_wrong_type(self):
-        with self.assertRaises(ValueError) as err:
-            self.service_local._do_action("test")
-        self.assertEqual("status_type should be enable/disable/start/stop/is-enabled/is-active/list-unit-files", str(err.exception))
+    @mock.patch('crmsh.utils.ServiceManager._call')
+    def test_start_service(self, mock_call: mock.MagicMock):
+        mock_call.return_value = ['node1']
+        self.assertEqual(['node1'], utils.ServiceManager.start_service('service1', remote_addr='node1'))
+        mock_call.assert_called_once_with('node1', [], "systemctl start 'service1'")
 
-    @mock.patch("crmsh.utils.get_stdout_stderr")
-    def test_do_action_except_run_error(self, mock_run):
-        mock_run.return_value = (1, None, "this command failed")
-        with self.assertRaises(ValueError) as err:
-            self.service_local._do_action("start")
-        self.assertEqual("Run \"systemctl start service1\" error: this command failed", str(err.exception))
-        mock_run.assert_called_once_with("systemctl start service1")
+    @mock.patch('crmsh.utils.ServiceManager._call')
+    def test_start_service_on_multiple_host(self, mock_call: mock.MagicMock):
+        mock_call.return_value = ['node1', 'node2']
+        self.assertEqual(['node1', 'node2'], utils.ServiceManager.start_service('service1', node_list=['node1', 'node2']))
+        mock_call.assert_called_once_with(None, ['node1', 'node2'], "systemctl start 'service1'")
 
-    @mock.patch("crmsh.utils.run_cmd_on_remote")
-    def test_do_action_remote(self, mock_run_remote):
-        mock_run_remote.return_value = (0, "data", None)
-        rc, out = self.service_remote._do_action("start")
-        assert rc == True
-        assert out == "data"
-        mock_run_remote.assert_called_once_with("systemctl start service1",
-                "node1",
-                "Run \"systemctl start service1\" on node1")
+    @mock.patch('crmsh.utils.ServiceManager._call')
+    def test_start_and_enable_service(self, mock_call: mock.MagicMock):
+        mock_call.return_value = ['node1']
+        self.assertEqual(['node1'], utils.ServiceManager.start_service('service1', enable=True, remote_addr='node1'))
+        mock_call.assert_called_once_with('node1', [], "systemctl enable --now 'service1'")
 
-    def test_is_available(self):
-        self.service_local._do_action = mock.Mock()
-        self.service_local._do_action.return_value = (True, "service1 service2")
-        assert self.service_local.is_available == True
-        self.service_local._do_action.assert_called_once_with("list-unit-files")
+    @mock.patch('crmsh.utils.ServiceManager._call')
+    def test_stop_service(self, mock_call: mock.MagicMock):
+        mock_call.return_value = ['node1']
+        self.assertEqual(['node1'], utils.ServiceManager.stop_service('service1', remote_addr='node1'))
+        mock_call.assert_called_once_with('node1', [], "systemctl stop 'service1'")
 
-    def test_is_enabled(self):
-        self.service_local._do_action = mock.Mock()
-        self.service_local._do_action.return_value = (True, None)
-        assert self.service_local.is_enabled == True
-        self.service_local._do_action.assert_called_once_with("is-enabled")
+    @mock.patch('crmsh.utils.ServiceManager._call')
+    def test_enable_service(self, mock_call: mock.MagicMock):
+        mock_call.return_value = ['node1']
+        self.assertEqual(['node1'], utils.ServiceManager.enable_service('service1', remote_addr='node1'))
+        mock_call.assert_called_once_with('node1', [], "systemctl enable 'service1'")
 
-    def test_is_active(self):
-        self.service_local._do_action = mock.Mock()
-        self.service_local._do_action.return_value = (True, None)
-        assert self.service_local.is_active == True
-        self.service_local._do_action.assert_called_once_with("is-active")
-
-    def test_start(self):
-        self.service_local._do_action = mock.Mock()
-        self.service_local._do_action.return_value = (True, None)
-        self.service_local.start()
-        self.service_local._do_action.assert_called_once_with("start")
-
-    def test_stop(self):
-        self.service_local._do_action = mock.Mock()
-        self.service_local._do_action.return_value = (True, None)
-        self.service_local.stop()
-        self.service_local._do_action.assert_called_once_with("stop")
-
-    def test_enable(self):
-        self.service_local._do_action = mock.Mock()
-        self.service_local._do_action.return_value = (True, None)
-        self.service_local.enable()
-        self.service_local._do_action.assert_called_once_with("enable")
-
-    def test_disable(self):
-        self.service_local._do_action = mock.Mock()
-        self.service_local._do_action.return_value = (True, None)
-        self.service_local.disable()
-        self.service_local._do_action.assert_called_once_with("disable")
-
-    @mock.patch("crmsh.utils.ServiceManager.is_available", new_callable=mock.PropertyMock)
-    def test_service_is_available(self, mock_available):
-        mock_available.return_value = True
-        res = utils.ServiceManager.service_is_available("service1")
-        self.assertEqual(res, True)
-        mock_available.assert_called_once_with()
-
-    @mock.patch("crmsh.utils.ServiceManager.is_enabled", new_callable=mock.PropertyMock)
-    def test_service_is_enabled(self, mock_enabled):
-        mock_enabled.return_value = True
-        res = utils.ServiceManager.service_is_enabled("service1")
-        self.assertEqual(res, True)
-        mock_enabled.assert_called_once_with()
-
-    @mock.patch("crmsh.utils.ServiceManager.is_active", new_callable=mock.PropertyMock)
-    def test_service_is_active(self, mock_active):
-        mock_active.return_value = True
-        res = utils.ServiceManager.service_is_active("service1")
-        self.assertEqual(res, True)
-        mock_active.assert_called_once_with()
-
-    @mock.patch('crmsh.utils.ServiceManager.start')
-    @mock.patch('crmsh.utils.ServiceManager.enable')
-    def test_start_service(self, mock_enable, mock_start):
-        utils.ServiceManager.start_service("service1", enable=True)
-        mock_enable.assert_called_once_with()
-        mock_start.assert_called_once_with()
-
-    @mock.patch('crmsh.utils.ServiceManager.stop')
-    @mock.patch('crmsh.utils.ServiceManager.disable')
-    def test_stop_service(self, mock_disable, mock_stop):
-        utils.ServiceManager.stop_service("service1", disable=True)
-        mock_disable.assert_called_once_with()
-        mock_stop.assert_called_once_with()
-
-    @mock.patch('crmsh.utils.ServiceManager.enable')
-    @mock.patch('crmsh.utils.ServiceManager.is_enabled', new_callable=mock.PropertyMock)
-    @mock.patch('crmsh.utils.ServiceManager.is_available', new_callable=mock.PropertyMock)
-    def test_enable_service(self, mock_available, mock_enabled, mock_enable):
-        mock_available.return_value = True
-        mock_enabled.return_value = False
-        utils.ServiceManager.enable_service("service1")
-        mock_enable.assert_called_once_with()
-
-    @mock.patch('crmsh.utils.ServiceManager.disable')
-    @mock.patch('crmsh.utils.ServiceManager.is_enabled', new_callable=mock.PropertyMock)
-    @mock.patch('crmsh.utils.ServiceManager.is_available', new_callable=mock.PropertyMock)
-    def test_disable_service(self, mock_available, mock_enabled, mock_disable):
-        mock_available.return_value = True
-        mock_enabled.return_value = True
-        utils.ServiceManager.disable_service("service1")
-        mock_disable.assert_called_once_with()
+    @mock.patch('crmsh.utils.ServiceManager._call')
+    def test_disable_service(self, mock_call: mock.MagicMock):
+        mock_call.return_value = ['node1']
+        self.assertEqual(['node1'], utils.ServiceManager.disable_service('service1', remote_addr='node1'))
+        mock_call.assert_called_once_with('node1', [], "systemctl disable 'service1'")
 
 
 @mock.patch("crmsh.utils.get_nodeid_from_name")
@@ -1217,21 +1127,182 @@ def test_calculate_quorate_status():
     assert utils.calculate_quorate_status(3, 1) is False
 
 
-@mock.patch("crmsh.utils.get_stdout_stderr")
-def test_get_stdout_or_raise_error_failed(mock_run):
-    mock_run.return_value = (1, None, "error data")
-    with pytest.raises(ValueError) as err:
-        utils.get_stdout_or_raise_error("cmd")
-    assert str(err.value) == 'Failed to run "cmd": error data'
-    mock_run.assert_called_once_with("cmd", no_reg=True)
+@mock.patch("crmsh.utils.subprocess_run_auto_ssh_no_input")
+@mock.patch("subprocess.run")
+def test_get_stdout_or_raise_error_local(
+        mock_subprocess_run: mock.MagicMock,
+        mock_subprocess_run_auto_ssh_no_input: mock.MagicMock,
+):
+    mock_subprocess_run.return_value = mock.Mock(returncode=0, stdout=b'bar', stderr=b'')
+    out = utils.get_stdout_or_raise_error("foo")
+    mock_subprocess_run.assert_called_once_with(
+        ['/bin/sh'],
+        input=b'foo',
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+    )
+    mock_subprocess_run_auto_ssh_no_input.assert_not_called()
+    assert "bar" == out
 
 
-@mock.patch("crmsh.utils.get_stdout_stderr")
-def test_get_stdout_or_raise_error(mock_run):
-    mock_run.return_value = (0, "output data", None)
-    res = utils.get_stdout_or_raise_error("cmd", remote="node1")
-    assert res == "output data"
-    mock_run.assert_called_once_with("ssh -o StrictHostKeyChecking=no root@node1 \"cmd\"", no_reg=True)
+@mock.patch("crmsh.utils.subprocess_run_auto_ssh_no_input")
+@mock.patch("subprocess.run")
+def test_get_stdout_or_raise_error_local_failure(
+        mock_subprocess_run: mock.MagicMock,
+        mock_subprocess_run_auto_ssh_no_input: mock.MagicMock,
+):
+    mock_subprocess_run.return_value = mock.Mock(returncode=1, stdout=b'bar', stderr=b'err')
+    exception = None
+    try:
+        out = utils.get_stdout_or_raise_error("foo")
+    except ValueError as e:
+        exception = e
+
+    mock_subprocess_run.assert_called_once_with(
+        ['/bin/sh'],
+        input=b'foo',
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+    )
+    mock_subprocess_run_auto_ssh_no_input.assert_not_called()
+    assert isinstance(exception, ValueError)
+
+
+@mock.patch("crmsh.utils.subprocess_run_auto_ssh_no_input")
+@mock.patch("subprocess.run")
+def test_get_stdout_or_raise_error_local_failure_no_raise(
+        mock_subprocess_run: mock.MagicMock,
+        mock_subprocess_run_auto_ssh_no_input: mock.MagicMock,
+):
+    mock_subprocess_run.return_value = mock.Mock(returncode=1, stdout=b'bar', stderr=b'err')
+    out = utils.get_stdout_or_raise_error("foo", no_raise=True)
+
+    mock_subprocess_run.assert_called_once_with(
+        ['/bin/sh'],
+        input=b'foo',
+        stdout=subprocess.PIPE,
+        stderr=subprocess.DEVNULL,
+    )
+    mock_subprocess_run_auto_ssh_no_input.assert_not_called()
+    assert "bar" == out
+
+
+@mock.patch("crmsh.utils.subprocess_run_auto_ssh_no_input")
+@mock.patch("subprocess.run")
+def test_get_stdout_or_raise_error_remote(
+        mock_subprocess_run: mock.MagicMock,
+        mock_subprocess_run_auto_ssh_no_input: mock.MagicMock,
+):
+    mock_subprocess_run_auto_ssh_no_input.return_value = mock.Mock(returncode=0, stdout=b'bar', stderr=b'')
+    out = utils.get_stdout_or_raise_error("foo", 'node1')
+    mock_subprocess_run.assert_not_called()
+    mock_subprocess_run_auto_ssh_no_input.assert_called_once_with(
+        "foo",
+        'node1',
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+    )
+    assert "bar" == out
+
+
+class TestSubprocessRunUtils(unittest.TestCase):
+    @mock.patch("subprocess.run")
+    def test_subprocess_run_auto_ssh_no_input_local_no_su(self, mock_subprocess_run):
+        mock_subprocess_run.return_value = mock.Mock(returncode=0, stdout=b'bar', stderr=b'')
+        result = utils.subprocess_run_auto_ssh_no_input("foo", stderr=subprocess.DEVNULL)
+        mock_subprocess_run.assert_called_once_with(['/bin/sh'], input=b'foo', stderr=subprocess.DEVNULL)
+        self.assertEqual(0, result.returncode)
+        self.assertEqual(b'bar', result.stdout)
+
+    @mock.patch("subprocess.run")
+    def test_subprocess_run_auto_ssh_no_input_local_su(self, mock_subprocess_run):
+        mock_subprocess_run.return_value = mock.Mock(returncode=0, stdout=b'bar', stderr=b'')
+        result = utils.subprocess_run_auto_ssh_no_input("foo", user="alice", stderr=subprocess.DEVNULL)
+        mock_subprocess_run.assert_called_once_with(
+            ['sudo', '-H', '-u', 'alice', '/bin/sh'],
+            input=b'foo',
+            stderr=subprocess.DEVNULL,
+        )
+        self.assertEqual(0, result.returncode)
+        self.assertEqual(b'bar', result.stdout)
+
+    @mock.patch("crmsh.utils.this_node")
+    @mock.patch('crmsh.utils.user_pair_for_ssh')
+    @mock.patch("crmsh.utils.su_subprocess_run")
+    @mock.patch("subprocess.run")
+    def test_subprocess_run_auto_ssh_no_input_remote_no_user(
+            self,
+            mock_subprocess_run: mock.MagicMock,
+            mock_su_subprocess_run: mock.MagicMock,
+            mock_user_pair_for_ssh: mock.MagicMock,
+            mock_this_node: mock.MagicMock,
+    ):
+        mock_this_node.return_value = 'node1'
+        mock_user_pair_for_ssh.return_value = "alice", "bob"
+        mock_su_subprocess_run.return_value = mock.Mock(returncode=0, stdout=b'bar', stderr=b'')
+        result = utils.subprocess_run_auto_ssh_no_input("foo", "node2", stderr=subprocess.DEVNULL)
+        mock_user_pair_for_ssh.assert_called_once_with('node2')
+        mock_subprocess_run.assert_not_called()
+        mock_su_subprocess_run.assert_called_once_with(
+            'alice',
+            'ssh -o StrictHostKeyChecking=no bob@node2 sudo -H -u root /bin/sh',
+            input=b'foo',
+            stderr=subprocess.DEVNULL,
+        )
+        self.assertEqual(0, result.returncode)
+        self.assertEqual(b'bar', result.stdout)
+
+    @mock.patch("os.geteuid")
+    @mock.patch("crmsh.userdir.getuser")
+    @mock.patch("subprocess.run")
+    def test_su_subprocess_run(
+            self,
+            mock_subprocess_run: mock.MagicMock,
+            mock_get_user: mock.MagicMock,
+            mock_geteuid: mock.MagicMock,
+    ):
+        mock_geteuid.return_value = 0
+        mock_get_user.return_value = 'root'
+        mock_subprocess_run.return_value = mock.Mock(returncode=0, stdout=b'bar', stderr=b'')
+        result = utils.su_subprocess_run('alice', 'foo')
+        mock_subprocess_run.assert_called_once_with(
+            ['su', 'alice', '--login', '-c', 'foo']
+        )
+        self.assertEqual(0, result.returncode)
+        self.assertEqual(b'bar', result.stdout)
+
+    @mock.patch("os.geteuid")
+    @mock.patch("crmsh.userdir.getuser")
+    @mock.patch("subprocess.run")
+    def test_su_subprocess_run_with_non_root_user(
+            self,
+            mock_subprocess_run: mock.MagicMock,
+            mock_get_user: mock.MagicMock,
+            mock_geteuid: mock.MagicMock,
+    ):
+        mock_geteuid.return_value = 1000
+        mock_get_user.return_value = 'bob'
+        with self.assertRaises(AssertionError):
+            result = utils.su_subprocess_run('alice', 'foo')
+
+    @mock.patch("os.geteuid")
+    @mock.patch("crmsh.userdir.getuser")
+    @mock.patch("subprocess.run")
+    def test_su_subprocess_run_to_self(
+            self,
+            mock_subprocess_run: mock.MagicMock,
+            mock_get_user: mock.MagicMock,
+            mock_geteuid: mock.MagicMock,
+    ):
+        mock_geteuid.return_value = 1000
+        mock_get_user.return_value = 'alice'
+        mock_subprocess_run.return_value = mock.Mock(returncode=0, stdout=b'bar', stderr=b'')
+        result = utils.su_subprocess_run('alice', 'foo')
+        mock_subprocess_run.assert_called_once_with(
+            ['/bin/sh', '-c', 'foo'],
+        )
+        self.assertEqual(0, result.returncode)
+        self.assertEqual(b'bar', result.stdout)
 
 
 @mock.patch("crmsh.utils.get_stdout_or_raise_error")
@@ -1564,7 +1635,18 @@ def test_list_cluster_nodes_none(mock_run, mock_etree):
     mock_etree.return_value = None
     res = utils.list_cluster_nodes()
     assert res is None
-    mock_run.assert_called_once_with(constants.CIB_QUERY)
+    mock_run.assert_called_once_with(constants.CIB_QUERY, no_reg=False)
+    mock_etree.assert_called_once_with("data")
+
+
+@mock.patch('crmsh.utils.etree.fromstring')
+@mock.patch('crmsh.utils.get_stdout_stderr')
+def test_list_cluster_nodes_none_no_reg(mock_run, mock_etree):
+    mock_run.return_value = (0, "data", None)
+    mock_etree.return_value = None
+    res = utils.list_cluster_nodes(no_reg=True)
+    assert res is None
+    mock_run.assert_called_once_with(constants.CIB_QUERY, no_reg=True)
     mock_etree.assert_called_once_with("data")
 
 
@@ -1577,7 +1659,7 @@ def test_list_cluster_nodes_cib_not_exist(mock_run, mock_env, mock_isfile):
     mock_isfile.return_value = False
     res = utils.list_cluster_nodes()
     assert res is None
-    mock_run.assert_called_once_with(constants.CIB_QUERY)
+    mock_run.assert_called_once_with(constants.CIB_QUERY, no_reg=False)
     mock_env.assert_called_once_with("CIB_file", constants.CIB_RAW_FILE)
     mock_isfile.assert_called_once_with(constants.CIB_RAW_FILE)
 
@@ -1601,7 +1683,7 @@ def test_list_cluster_nodes(mock_run, mock_env, mock_isfile, mock_file2elem):
     res = utils.list_cluster_nodes()
     assert res == ["node2"]
 
-    mock_run.assert_called_once_with(constants.CIB_QUERY)
+    mock_run.assert_called_once_with(constants.CIB_QUERY, no_reg=False)
     mock_env.assert_called_once_with("CIB_file", constants.CIB_RAW_FILE)
     mock_isfile.assert_called_once_with(constants.CIB_RAW_FILE)
     mock_file2elem.assert_called_once_with(constants.CIB_RAW_FILE)
@@ -1611,17 +1693,40 @@ def test_list_cluster_nodes(mock_run, mock_env, mock_isfile, mock_file2elem):
         ])
 
 
+@mock.patch('os.getenv')
 @mock.patch('crmsh.utils.get_stdout_stderr')
-def test_get_property(mock_run):
+def test_get_property(mock_run, mock_env):
     mock_run.return_value = (0, "data", None)
+    mock_env.return_value = "cib.xml"
     assert utils.get_property("no-quorum-policy") == "data"
-    mock_run.assert_called_once_with("crm configure get_property no-quorum-policy")
+    mock_run.assert_called_once_with("CIB_file=cib.xml sudo --preserve-env=CIB_file crm configure get_property no-quorum-policy")
 
 
+@mock.patch('logging.Logger.warning')
 @mock.patch('crmsh.utils.get_stdout_or_raise_error')
-def test_set_property(mock_run):
-    utils.set_property(no_quorum_policy="stop")
+@mock.patch('crmsh.utils.get_property')
+def test_set_property(mock_get, mock_run, mock_warn):
+    mock_get.return_value = "start"
+    utils.set_property("no-quorum-policy", "stop")
     mock_run.assert_called_once_with("crm configure property no-quorum-policy=stop")
+    mock_warn.assert_called_once_with('"no-quorum-policy" in crm_config is set to stop, it was start')
+
+
+@mock.patch('crmsh.utils.get_property')
+def test_set_property_the_same(mock_get):
+    mock_get.return_value = "value1"
+    utils.set_property("no-quorum-policy", "value1")
+    mock_get.assert_called_once_with("no-quorum-policy", "crm_config")
+
+
+@mock.patch('crmsh.utils.crm_msec')
+@mock.patch('crmsh.utils.get_property')
+def test_set_property_conditional(mock_get, mock_msec):
+    mock_get.return_value = "10s"
+    mock_msec.side_effect = ["1000", "1000"]
+    utils.set_property("timeout", "10", conditional=True)
+    mock_get.assert_called_once_with("timeout", "crm_config")
+    mock_msec.assert_has_calls([mock.call("10s"), mock.call("10")])
 
 
 @mock.patch('crmsh.utils.is_dlm_configured')
@@ -1659,14 +1764,6 @@ def test_get_systemd_timeout_start_in_sec():
     assert res == 91
 
 
-@mock.patch('crmsh.utils.get_stdout_or_raise_error')
-@mock.patch('crmsh.utils.get_property')
-def test_set_property_conditionally(mock_get_property, mock_run):
-    mock_get_property.return_value = "100s"
-    utils.set_property_conditionally("stonith-timeout", 101)
-    mock_run.assert_called_once_with("crm configure property stonith-timeout=101")
-
-
 @mock.patch('crmsh.utils.is_larger_than_min_version')
 @mock.patch('crmsh.cibconfig.cib_factory')
 def test_is_ocf_1_1_cib_schema_detected(mock_cib, mock_larger):
@@ -1707,3 +1804,122 @@ def test_handle_role_for_ocf_1_1_return(mock_support):
 
 def test_handle_role_for_ocf_1_1_return_not_role():
     assert utils.handle_role_for_ocf_1_1("test", name='other') == "test"
+
+
+def test_compatible_role():
+    assert utils.compatible_role("Slave", "Unpromoted") is True
+
+
+@mock.patch('logging.Logger.warning')
+@mock.patch('crmsh.utils.get_stdout_or_raise_error')
+def test_fetch_cluster_node_list_from_node(mock_run, mock_warn):
+    mock_run.return_value = """
+
+    1 node1
+    2 node2 lost
+    3 node3 member
+    """
+    assert utils.fetch_cluster_node_list_from_node("node1") == ["node3"]
+    mock_run.assert_called_once_with("crm_node -l", remote="node1")
+    mock_warn.assert_has_calls([
+        mock.call("The node '%s' has no known name and/or state information", "1"),
+        mock.call("The node '%s'(state '%s') is not a current member", "node2", "lost")
+        ])
+
+
+@mock.patch('crmsh.utils.list_cluster_nodes_except_me')
+def test_cluster_copy_file_return(mock_list_nodes):
+    mock_list_nodes.return_value = []
+    assert utils.cluster_copy_file("/file1") == True
+
+
+@mock.patch('crmsh.utils.get_stdout_stderr')
+def test_has_sudo_access(mock_run):
+    mock_run.return_value = (0, None, None)
+    assert utils.has_sudo_access() is True
+    mock_run.assert_called_once_with("sudo -S -k -n id -u")
+
+
+@mock.patch('grp.getgrgid')
+@mock.patch('os.getgroups')
+def test_in_haclient(mock_group, mock_getgrgid):
+    mock_group.return_value = [90, 100]
+    mock_getgrgid_inst1 = mock.Mock(gr_name=constants.HA_GROUP)
+    mock_getgrgid_inst2 = mock.Mock(gr_name="other")
+    mock_getgrgid.side_effect = [mock_getgrgid_inst1, mock_getgrgid_inst2]
+    assert utils.in_haclient() is True
+    mock_group.assert_called_once_with()
+
+
+@mock.patch('crmsh.utils.in_haclient')
+@mock.patch('crmsh.userdir.getuser')
+def test_check_user_access_root(mock_user, mock_in):
+    mock_user.return_value = 'root'
+    utils.check_user_access('cluster')
+    mock_in.assert_not_called()
+
+
+@mock.patch('crmsh.utils.has_sudo_access')
+@mock.patch('crmsh.utils.in_haclient')
+@mock.patch('crmsh.userdir.getuser')
+def test_check_user_access_haclient(mock_user, mock_in, mock_sudo):
+    mock_user.return_value = 'user'
+    mock_in.return_value = True
+    utils.check_user_access('ra')
+    mock_sudo.assert_not_called()
+
+
+@mock.patch('logging.Logger.error')
+@mock.patch('crmsh.utils.has_sudo_access')
+@mock.patch('crmsh.utils.in_haclient')
+@mock.patch('crmsh.userdir.getuser')
+def test_check_user_access_need_sudo(mock_user, mock_in, mock_sudo, mock_error):
+    mock_user.return_value = 'user'
+    mock_in.return_value = False
+    mock_sudo.return_value = True
+    with pytest.raises(utils.TerminateSubCommand) as err:
+        utils.check_user_access('ra')
+    mock_error.assert_called_once_with('Please run this command starting with "sudo"')
+
+
+@mock.patch('logging.Logger.error')
+@mock.patch('crmsh.utils.has_sudo_access')
+@mock.patch('crmsh.utils.in_haclient')
+@mock.patch('crmsh.userdir.getuser')
+def test_check_user_access_acl(mock_user, mock_in, mock_sudo, mock_error):
+    mock_user.return_value = 'user'
+    mock_in.return_value = False
+    mock_sudo.return_value = False
+    with pytest.raises(utils.TerminateSubCommand) as err:
+        utils.check_user_access('ra')
+    mock_error.assert_called_once_with('This command needs higher privilege.\nOption 1) Please consider to add "user" as sudoer. For example:\n  sudo bash -c \'echo "user ALL=(ALL) NOPASSWD:ALL" > /etc/sudoers.d/user\'\nOption 2) Add "user" to the haclient group. For example:\n  sudo usermod -g haclient user')
+
+
+@mock.patch('logging.Logger.error')
+@mock.patch('crmsh.utils.has_sudo_access')
+@mock.patch('crmsh.utils.in_haclient')
+@mock.patch('crmsh.userdir.getuser')
+def test_check_user_access_cluster(mock_user, mock_in, mock_sudo, mock_error):
+    mock_user.return_value = 'user'
+    mock_in.return_value = False
+    mock_sudo.return_value = False
+    with pytest.raises(utils.TerminateSubCommand) as err:
+        utils.check_user_access('cluster')
+    mock_error.assert_called_once_with('Please run this command starting with "sudo".\nCurrently, this command needs to use sudo to escalate itself as root.\nPlease consider to add "user" as sudoer. For example:\n  sudo bash -c \'echo "user ALL=(ALL) NOPASSWD:ALL" > /etc/sudoers.d/user\'')
+
+
+@mock.patch('logging.Logger.debug')
+@mock.patch('crmsh.utils.get_stdout_stderr')
+@mock.patch('crmsh.utils.check_ssh_passwd_need')
+@mock.patch('crmsh.utils.this_node')
+def test_check_passwordless_between_nodes(mock_this_node, mock_check_ssh, mock_run, mock_debug):
+    mock_this_node.return_value = "node1"
+    mock_check_ssh.return_value = False
+    mock_run.return_value = (1, None, None)
+
+    res_list = utils.check_passwordless_between_nodes(["node1", "node2"])
+    assert res_list == [("node2", "node1")]
+
+    mock_run.assert_called_once_with('ssh node2 "ssh -o StrictHostKeyChecking=no -T -o Batchmode=yes root@node1 true"')
+    mock_debug.assert_called_once_with("There is no passwordless configured from %s to %s under '%s'", 'node2', 'node1', 'root')
+
